@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,23 +20,24 @@
 require 'uri'
 require 'cgi'
 
-class Unauthorized < Exception; end
+class Unauthorized < StandardError; end
 
 class ApplicationController < ActionController::Base
   include Redmine::I18n
   include Redmine::Pagination
   include Redmine::Hook::Helper
   include RoutesHelper
+  include AvatarsHelper
+
   helper :routes
+  helper :avatars
 
   class_attribute :accept_api_auth_actions
-  class_attribute :accept_rss_auth_actions
+  class_attribute :accept_atom_auth_actions
   class_attribute :model_object
 
   layout 'base'
 
-  protect_from_forgery
-
   def verify_authenticity_token
     unless api_request?
       super
@@ -43,15 +46,21 @@ class ApplicationController < ActionController::Base
 
   def handle_unverified_request
     unless api_request?
-      super
-      cookies.delete(autologin_cookie_name)
-      self.logged_user = nil
-      set_localization
-      render_error :status => 422, :message => "Invalid form authenticity token."
+      begin
+        super
+      rescue ActionController::InvalidAuthenticityToken => e
+        logger.error("ActionController::InvalidAuthenticityToken: #{e.message}") if logger
+      ensure
+        cookies.delete(autologin_cookie_name)
+        self.logged_user = nil
+        set_localization
+        render_error :status => 422, :message => l(:error_invalid_authenticity_token)
+      end
     end
   end
 
-  before_action :session_expiration, :user_setup, :check_if_login_required, :set_localization, :check_password_change
+  before_action :session_expiration, :user_setup, :check_if_login_required, :set_localization, :check_password_change, :check_twofa_activation
+  after_action :record_project_usage
 
   rescue_from ::Unauthorized, :with => :deny_access
   rescue_from ::ActionView::MissingTemplate, :with => :missing_template
@@ -83,6 +92,9 @@ class ApplicationController < ActionController::Base
     if user.must_change_password?
       session[:pwd] = '1'
     end
+    if user.must_activate_twofa?
+      session[:must_activate_twofa] = '1'
+    end
   end
 
   def user_setup
@@ -100,22 +112,34 @@ class ApplicationController < ActionController::Base
     unless api_request?
       if session[:user_id]
         # existing session
-        user = (User.active.find(session[:user_id]) rescue nil)
+        user =
+          begin
+            User.active.find(session[:user_id])
+          rescue
+            nil
+          end
       elsif autologin_user = try_to_autologin
         user = autologin_user
-      elsif params[:format] == 'atom' && params[:key] && request.get? && accept_rss_auth?
-        # RSS key authentication does not start a session
-        user = User.find_by_rss_key(params[:key])
+      elsif params[:format] == 'atom' && params[:key] && request.get? && accept_atom_auth?
+        # ATOM key authentication does not start a session
+        user = User.find_by_atom_key(params[:key])
       end
     end
     if user.nil? && Setting.rest_api_enabled? && accept_api_auth?
       if (key = api_key_from_request)
         # Use API key
         user = User.find_by_api_key(key)
-      elsif request.authorization.to_s =~ /\ABasic /i
+      elsif /\ABasic /i.match?(request.authorization.to_s)
         # HTTP Basic, either username/password or API key/random
         authenticate_with_http_basic do |username, password|
-          user = User.try_to_login(username, password) || User.find_by_api_key(username)
+          user = User.try_to_login(username, password)
+          # Don't allow using username/password when two-factor auth is active
+          if user&.twofa_active?
+            render_error :message => 'HTTP Basic authentication is not allowed. Use API key instead', :status => 401
+            return
+          end
+
+          user ||= User.find_by_api_key(username)
         end
         if user && user.must_change_password?
           render_error :message => 'You must change your password', :status => 403
@@ -180,6 +204,7 @@ class ApplicationController < ActionController::Base
   def check_if_login_required
     # no check needed if user is already logged in
     return true if User.current.logged?
+
     require_login if Setting.login_required?
   end
 
@@ -194,6 +219,31 @@ class ApplicationController < ActionController::Base
     end
   end
 
+  def init_twofa_pairing_and_send_code_for(twofa)
+    twofa.init_pairing!
+    if twofa.send_code(controller: 'twofa', action: 'activate')
+      flash[:notice] = l('twofa_code_sent')
+    end
+    redirect_to controller: 'twofa', action: 'activate_confirm', scheme: twofa.scheme_name
+  end
+
+  def check_twofa_activation
+    if session[:must_activate_twofa]
+      if User.current.must_activate_twofa?
+        flash[:warning] = l('twofa_warning_require')
+        if Redmine::Twofa.available_schemes.length == 1
+          twofa_scheme = Redmine::Twofa.for_twofa_scheme(Redmine::Twofa.available_schemes.first)
+          twofa = twofa_scheme.new(User.current)
+          init_twofa_pairing_and_send_code_for(twofa)
+        else
+          redirect_to controller: 'twofa', action: 'select_scheme'
+        end
+      else
+        session.delete(:must_activate_twofa)
+      end
+    end
+  end
+
   def set_localization(user=User.current)
     lang = nil
     if user && user.logged?
@@ -219,20 +269,25 @@ class ApplicationController < ActionController::Base
         url = url_for(:controller => params[:controller], :action => params[:action], :id => params[:id], :project_id => params[:project_id])
       end
       respond_to do |format|
-        format.html {
+        format.html do
           if request.xhr?
             head :unauthorized
           else
             redirect_to signin_path(:back_url => url)
           end
-        }
-        format.any(:atom, :pdf, :csv) {
+        end
+        format.any(:atom, :pdf, :csv) do
           redirect_to signin_path(:back_url => url)
-        }
-        format.xml  { head :unauthorized, 'WWW-Authenticate' => 'Basic realm="Redmine API"' }
-        format.js   { head :unauthorized, 'WWW-Authenticate' => 'Basic realm="Redmine API"' }
-        format.json { head :unauthorized, 'WWW-Authenticate' => 'Basic realm="Redmine API"' }
-        format.any  { head :unauthorized }
+        end
+        format.api do
+          if Setting.rest_api_enabled? && accept_api_auth?
+            head(:unauthorized, 'WWW-Authenticate' => 'Basic realm="Redmine API"')
+          else
+            head(:forbidden)
+          end
+        end
+        format.js   {head :unauthorized, 'WWW-Authenticate' => 'Basic realm="Redmine API"'}
+        format.any  {head :unauthorized}
       end
       return false
     end
@@ -241,6 +296,7 @@ class ApplicationController < ActionController::Base
 
   def require_admin
     return unless require_login
+
     if !User.current.admin?
       render_403
       return false
@@ -331,6 +387,7 @@ class ApplicationController < ActionController::Base
     # if the issue actually exists but requires authentication
     @issue = Issue.find(params[:id])
     raise Unauthorized unless @issue.visible?
+
     @project = @issue.project
   rescue ActiveRecord::RecordNotFound
     render_404
@@ -341,10 +398,13 @@ class ApplicationController < ActionController::Base
   def find_issues
     @issues = Issue.
       where(:id => (params[:id] || params[:ids])).
-      preload(:project, :status, :tracker, :priority, :author, :assigned_to, :relations_to, {:custom_values => :custom_field}).
+      preload(:project, :status, :tracker, :priority,
+              :author, :assigned_to, :relations_to,
+              {:custom_values => :custom_field}).
       to_a
     raise ActiveRecord::RecordNotFound if @issues.empty?
     raise Unauthorized unless @issues.all?(&:visible?)
+
     @projects = @issues.collect(&:project).compact.uniq
     @project = @projects.first if @projects.size == 1
   rescue ActiveRecord::RecordNotFound
@@ -354,7 +414,7 @@ class ApplicationController < ActionController::Base
   def find_attachments
     if (attachments = params[:attachments]).present?
       att = attachments.values.collect do |attachment|
-        Attachment.find_by_token( attachment[:token] ) if attachment[:token].present?
+        Attachment.find_by_token(attachment[:token]) if attachment[:token].present?
       end
       att.compact!
     end
@@ -362,11 +422,19 @@ class ApplicationController < ActionController::Base
   end
 
   def parse_params_for_bulk_update(params)
-    attributes = (params || {}).reject {|k,v| v.blank?}
-    attributes.keys.each {|k| attributes[k] = '' if attributes[k] == 'none'}
+    attributes = (params || {}).reject {|k, v| v.blank?}
     if custom = attributes[:custom_field_values]
-      custom.reject! {|k,v| v.blank?}
-      custom.keys.each do |k|
+      custom.reject! {|k, v| v.blank?}
+    end
+
+    replace_none_values_with_blank(attributes)
+  end
+
+  def replace_none_values_with_blank(params)
+    attributes = (params || {})
+    attributes.each_key {|k| attributes[k] = '' if attributes[k] == 'none'}
+    if (custom = attributes[:custom_field_values])
+      custom.each_key do |k|
         if custom[k].is_a?(Array)
           custom[k] << '' if custom[k].delete('__none__')
         else
@@ -393,6 +461,13 @@ class ApplicationController < ActionController::Base
     end
   end
 
+  def record_project_usage
+    if @project && @project.id && User.current.logged? && User.current.allowed_to?(:view_project, @project)
+      Redmine::ProjectJumpBox.new(User.current).project_used(@project)
+    end
+    true
+  end
+
   def back_url
     url = params[:back_url]
     if url.nil? && referer = request.env['HTTP_REFERER']
@@ -438,6 +513,7 @@ class ApplicationController < ActionController::Base
       if uri.send(component).present? && uri.send(component) != request.send(component)
         return false
       end
+
       uri.send(:"#{component}=", nil)
     end
     # Always ignore basic user:password in the URL
@@ -446,11 +522,11 @@ class ApplicationController < ActionController::Base
     path = uri.to_s
     # Ensure that the remaining URL starts with a slash, followed by a
     # non-slash character or the end
-    if path !~ %r{\A/([^/]|\z)}
+    if !%r{\A/([^/]|\z)}.match?(path)
       return false
     end
 
-    if path.match(%r{/(login|account/register|account/lost_password)})
+    if %r{/(login|account/register|account/lost_password)}.match?(path)
       return false
     end
 
@@ -477,7 +553,7 @@ class ApplicationController < ActionController::Base
       if args.any?
         redirect_to *args
       elsif block_given?
-        block.call
+        yield
       else
         raise "#redirect_to_referer_or takes arguments or a block"
       end
@@ -504,10 +580,10 @@ class ApplicationController < ActionController::Base
     @status = arg[:status] || 500
 
     respond_to do |format|
-      format.html {
+      format.html do
         render :template => 'common/error', :layout => use_layout, :status => @status
-      }
-      format.any { head @status }
+      end
+      format.any {head @status}
     end
   end
 
@@ -522,6 +598,7 @@ class ApplicationController < ActionController::Base
   # but have no HTML representation for non admin users
   def require_admin_or_api_request
     return true if api_request?
+
     if User.current.admin?
       true
     elsif User.current.logged?
@@ -540,23 +617,34 @@ class ApplicationController < ActionController::Base
 
   def render_feed(items, options={})
     @items = (items || []).to_a
-    @items.sort! {|x,y| y.event_datetime <=> x.event_datetime }
+    @items.sort! {|x, y| y.event_datetime <=> x.event_datetime}
     @items = @items.slice(0, Setting.feeds_limit.to_i)
     @title = options[:title] || Setting.app_title
     render :template => "common/feed", :formats => [:atom], :layout => false,
            :content_type => 'application/atom+xml'
   end
 
-  def self.accept_rss_auth(*actions)
+  def self.accept_atom_auth(*actions)
     if actions.any?
-      self.accept_rss_auth_actions = actions
+      self.accept_atom_auth_actions = actions
     else
-      self.accept_rss_auth_actions || []
+      self.accept_atom_auth_actions || []
     end
   end
 
+  def self.accept_rss_auth(*actions)
+    ActiveSupport::Deprecation.warn "Application#self.accept_rss_auth is deprecated and will be removed in Redmine 6.0. Please use #self.accept_atom_auth instead."
+    self.class.accept_atom_auth(*actions)
+  end
+
+  def accept_atom_auth?(action=action_name)
+    self.class.accept_atom_auth.include?(action.to_sym)
+  end
+
+  # TODO: remove in Redmine 6.0
   def accept_rss_auth?(action=action_name)
-    self.class.accept_rss_auth.include?(action.to_sym)
+    ActiveSupport::Deprecation.warn "Application#accept_rss_auth? is deprecated and will be removed in Redmine 6.0. Please use #accept_atom_auth? instead."
+    accept_atom_auth?(action)
   end
 
   def self.accept_api_auth(*actions)
@@ -616,13 +704,13 @@ class ApplicationController < ActionController::Base
     tmp = []
     if value
       parts = value.split(/,\s*/)
-      parts.each {|part|
+      parts.each do |part|
         if m = %r{^([^\s,]+?)(?:;\s*q=(\d+(?:\.\d+)?))?$}.match(part)
           val = m[1]
           q = (m[2] or 1).to_f
           tmp.push([val, q])
         end
-      }
+      end
       tmp = tmp.sort_by{|val, q| -q}
       tmp.collect!{|val, q| val}
     end
@@ -633,7 +721,7 @@ class ApplicationController < ActionController::Base
 
   # Returns a string that can be used as filename value in Content-Disposition header
   def filename_for_content_disposition(name)
-    request.env['HTTP_USER_AGENT'] =~ %r{(MSIE|Trident|Edge)} ? ERB::Util.url_encode(name) : name
+    name
   end
 
   def api_request?
@@ -663,12 +751,19 @@ class ApplicationController < ActionController::Base
   def query_statement_invalid(exception)
     logger.error "Query::StatementInvalid: #{exception.message}" if logger
     session.delete(:issue_query)
-    render_error "An error occurred while executing the query and has been logged. Please report this error to your Redmine administrator."
+    render_error l(:error_query_statement_invalid)
   end
 
-  # Renders a 200 response for successful updates or deletions via the API
+  def query_error(exception)
+    Rails.logger.debug "#{exception.class.name}: #{exception.message}"
+    Rails.logger.debug "    #{exception.backtrace.join("\n    ")}"
+
+    render_404
+  end
+
+  # Renders a 204 response for successful updates or deletions via the API
   def render_api_ok
-    render_api_head :ok
+    render_api_head :no_content
   end
 
   # Renders a head API response
@@ -685,7 +780,7 @@ class ApplicationController < ActionController::Base
 
   def render_api_errors(*messages)
     @error_messages = messages.flatten
-    render :template => 'common/error_messages.api', :status => :unprocessable_entity, :layout => nil
+    render :template => 'common/error_messages', :format => [:api], :status => :unprocessable_entity, :layout => nil
   end
 
   # Overrides #_include_layout? so that #render with no arguments
