@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -56,9 +58,11 @@ class Mailer < ActionMailer::Base
     options = {:protocol => Setting.protocol}
     if Setting.host_name.to_s =~ /\A(https?\:\/\/)?(.+?)(\:(\d+))?(\/.+)?\z/i
       host, port, prefix = $2, $4, $5
-      options.merge!({
-        :host => host, :port => port, :script_name => prefix
-      })
+      options.merge!(
+        {
+          :host => host, :port => port, :script_name => prefix
+        }
+      )
     else
       options[:host] = Setting.host_name
     end
@@ -68,17 +72,21 @@ class Mailer < ActionMailer::Base
   # Builds a mail for notifying user about a new issue
   def issue_add(user, issue)
     redmine_headers 'Project' => issue.project.identifier,
+                    'Issue-Tracker' => issue.tracker.name,
                     'Issue-Id' => issue.id,
-                    'Issue-Author' => issue.author.login
-    redmine_headers 'Issue-Assignee' => issue.assigned_to.login if issue.assigned_to
+                    'Issue-Author' => issue.author.login,
+                    'Issue-Assignee' => assignee_for_header(issue)
     message_id issue
     references issue
     @author = issue.author
     @issue = issue
     @user = user
     @issue_url = url_for(:controller => 'issues', :action => 'show', :id => issue)
+    subject = "[#{issue.project.name} - #{issue.tracker.name} ##{issue.id}]"
+    subject += " (#{issue.status.name})" if Setting.show_status_changes_in_mail_subject?
+    subject += " #{issue.subject}"
     mail :to => user,
-      :subject => "[#{issue.project.name} - #{issue.tracker.name} ##{issue.id}] (#{issue.status.name}) #{issue.subject}"
+      :subject => subject
   end
 
   # Notifies users about a new issue.
@@ -86,7 +94,7 @@ class Mailer < ActionMailer::Base
   # Example:
   #   Mailer.deliver_issue_add(issue)
   def self.deliver_issue_add(issue)
-    users = issue.notified_users | issue.notified_watchers
+    users = issue.notified_users | issue.notified_watchers | issue.notified_mentions
     users.each do |user|
       issue_add(user, issue).deliver_later
     end
@@ -96,15 +104,16 @@ class Mailer < ActionMailer::Base
   def issue_edit(user, journal)
     issue = journal.journalized
     redmine_headers 'Project' => issue.project.identifier,
+                    'Issue-Tracker' => issue.tracker.name,
                     'Issue-Id' => issue.id,
-                    'Issue-Author' => issue.author.login
-    redmine_headers 'Issue-Assignee' => issue.assigned_to.login if issue.assigned_to
+                    'Issue-Author' => issue.author.login,
+                    'Issue-Assignee' => assignee_for_header(issue)
     message_id journal
     references issue
     @author = journal.user
     s = "[#{issue.project.name} - #{issue.tracker.name} ##{issue.id}] "
-    s << "(#{issue.status.name}) " if journal.new_value_for('status_id')
-    s << issue.subject
+    s += "(#{issue.status.name}) " if journal.new_value_for('status_id') && Setting.show_status_changes_in_mail_subject?
+    s += issue.subject
     @issue = issue
     @user = user
     @journal = journal
@@ -120,7 +129,7 @@ class Mailer < ActionMailer::Base
   # Example:
   #   Mailer.deliver_issue_edit(journal)
   def self.deliver_issue_edit(journal)
-    users  = journal.notified_users | journal.notified_watchers
+    users  = journal.notified_users | journal.notified_watchers | journal.notified_mentions | journal.journalized.notified_mentions
     users.select! do |user|
       journal.notes? || journal.visible_details(user).any?
     end
@@ -134,6 +143,7 @@ class Mailer < ActionMailer::Base
     redmine_headers 'Project' => document.project.identifier
     @author = author
     @document = document
+    @user = user
     @document_url = url_for(:controller => 'documents', :action => 'show', :id => document)
     mail :to => user,
       :subject => "[#{document.project.name}] #{l(:label_document_new)}: #{document.title}"
@@ -169,6 +179,7 @@ class Mailer < ActionMailer::Base
     end
     redmine_headers 'Project' => container.project.identifier
     @attachments = attachments
+    @user = user
     @added_to = added_to
     @added_to_url = added_to_url
     mail :to => user,
@@ -200,6 +211,7 @@ class Mailer < ActionMailer::Base
     message_id news
     references news
     @news = news
+    @user = user
     @news_url = url_for(:controller => 'news', :action => 'show', :id => news)
     mail :to => user,
       :subject => "[#{news.project.name}] #{l(:label_news)}: #{news.title}"
@@ -225,6 +237,7 @@ class Mailer < ActionMailer::Base
     references news
     @news = news
     @comment = comment
+    @user = user
     @news_url = url_for(:controller => 'news', :action => 'show', :id => news)
     mail :to => user,
      :subject => "Re: [#{news.project.name}] #{l(:label_news)}: #{news.title}"
@@ -250,6 +263,7 @@ class Mailer < ActionMailer::Base
     message_id message
     references message.root
     @message = message
+    @user = user
     @message_url = url_for(message.event_url)
     mail :to => user,
       :subject => "[#{message.board.project.name} - #{message.board.name} - msg#{message.root.id}] #{message.subject}"
@@ -276,11 +290,15 @@ class Mailer < ActionMailer::Base
     @author = wiki_content.author
     message_id wiki_content
     @wiki_content = wiki_content
+    @user = user
     @wiki_content_url = url_for(:controller => 'wiki', :action => 'show',
                                       :project_id => wiki_content.project,
                                       :id => wiki_content.page.title)
-    mail :to => user,
-      :subject => "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_added, :id => wiki_content.page.pretty_title)}"
+    mail(
+      :to => user,
+      :subject =>
+        "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_added, :id => wiki_content.page.pretty_title)}"
+    )
   end
 
   # Notifies users about a new wiki content (wiki page added).
@@ -288,7 +306,7 @@ class Mailer < ActionMailer::Base
   # Example:
   #   Mailer.deliver_wiki_content_added(wiki_content)
   def self.deliver_wiki_content_added(wiki_content)
-    users = wiki_content.notified_users | wiki_content.page.wiki.notified_watchers
+    users = wiki_content.notified_users | wiki_content.page.wiki.notified_watchers | wiki_content.notified_mentions
     users.each do |user|
       wiki_content_added(user, wiki_content).deliver_later
     end
@@ -301,14 +319,20 @@ class Mailer < ActionMailer::Base
     @author = wiki_content.author
     message_id wiki_content
     @wiki_content = wiki_content
-    @wiki_content_url = url_for(:controller => 'wiki', :action => 'show',
-                                      :project_id => wiki_content.project,
-                                      :id => wiki_content.page.title)
-    @wiki_diff_url = url_for(:controller => 'wiki', :action => 'diff',
-                                   :project_id => wiki_content.project, :id => wiki_content.page.title,
-                                   :version => wiki_content.version)
-    mail :to => user,
-      :subject => "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_updated, :id => wiki_content.page.pretty_title)}"
+    @user = user
+    @wiki_content_url =
+      url_for(:controller => 'wiki', :action => 'show',
+              :project_id => wiki_content.project,
+              :id => wiki_content.page.title)
+    @wiki_diff_url =
+      url_for(:controller => 'wiki', :action => 'diff',
+              :project_id => wiki_content.project, :id => wiki_content.page.title,
+              :version => wiki_content.version)
+    mail(
+      :to => user,
+      :subject =>
+        "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_updated, :id => wiki_content.page.pretty_title)}"
+    )
   end
 
   # Notifies users about the update of the specified wiki content
@@ -319,6 +343,7 @@ class Mailer < ActionMailer::Base
     users  = wiki_content.notified_users
     users |= wiki_content.page.notified_watchers
     users |= wiki_content.page.wiki.notified_watchers
+    users |= wiki_content.notified_mentions
 
     users.each do |user|
       wiki_content_updated(user, wiki_content).deliver_later
@@ -391,8 +416,8 @@ class Mailer < ActionMailer::Base
   # The email will be sent to the email address specifiedby recipient if provided.
   #
   # Exemple:
-  #   Mailer.deliver_account_activated(user, token)
-  #   Mailer.deliver_account_activated(user, token, 'foo@example.net')
+  #   Mailer.deliver_lost_password(user, token)
+  #   Mailer.deliver_lost_password(user, token, 'foo@example.net')
   def self.deliver_lost_password(user, token, recipient=nil)
     lost_password(user, token, recipient).deliver_later
   end
@@ -407,7 +432,8 @@ class Mailer < ActionMailer::Base
     # TODO: maybe not the best way to handle this
     return if user.admin? && user.login == 'admin' && user.mail == 'admin@example.net'
 
-    deliver_security_notification(user,
+    deliver_security_notification(
+      user,
       sender,
       message: :mail_body_password_updated,
       title: :button_change_password,
@@ -444,10 +470,10 @@ class Mailer < ActionMailer::Base
   def security_notification(user, sender, options={})
     @sender = sender
     redmine_headers 'Sender' => sender.login
-    @message = l(options[:message],
-      field: (options[:field] && l(options[:field])),
-      value: options[:value]
-    )
+    @message =
+      l(options[:message],
+        field: (options[:field] && l(options[:field])),
+        value: options[:value])
     @title = options[:title] && l(options[:title])
     @remote_ip = options[:remote_ip] || @sender.remote_ip
     @url = options[:url] && (options[:url].is_a?(Hash) ? url_for(options[:url]) : options[:url])
@@ -472,7 +498,7 @@ class Mailer < ActionMailer::Base
   def self.deliver_security_notification(users, sender, options={})
     # Symbols cannot be serialized:
     # ActiveJob::SerializationError: Unsupported argument type: Symbol
-    options = options.transform_values {|v| v.is_a?(Symbol) ? v.to_s : v }
+    options = options.transform_values {|v| v.is_a?(Symbol) ? v.to_s : v}
     # sender's remote_ip would be lost on serialization/deserialization
     # we have to pass it with options
     options[:remote_ip] ||= sender.remote_ip
@@ -538,9 +564,18 @@ class Mailer < ActionMailer::Base
   def reminder(user, issues, days)
     @issues = issues
     @days = days
-    @issues_url = url_for(:controller => 'issues', :action => 'index',
+    @open_issues_url = url_for(:controller => 'issues', :action => 'index',
                                 :set_filter => 1, :assigned_to_id => 'me',
                                 :sort => 'due_date:asc')
+    @reminder_issues_url = url_for(:controller => 'issues', :action => 'index',
+      :set_filter => 1, :sort => 'due_date:asc',
+      :f => ['status_id', 'assigned_to_id', "due_date"],
+      :op => {'status_id' => 'o', 'assigned_to_id' => '=', 'due_date' => '<t+'},
+      :v =>{'assigned_to_id' => ['me'], 'due_date' => [days]})
+
+    query = IssueQuery.new(:name => '_')
+    query.add_filter('assigned_to_id', '=', ['me'])
+    @open_issues_count = query.issue_count
     mail :to => user,
       :subject => l(:mail_subject_reminder, :count => issues.size, :days => days)
   end
@@ -560,12 +595,15 @@ class Mailer < ActionMailer::Base
     if options[:version] && target_version_id.blank?
       raise ActiveRecord::RecordNotFound.new("Couldn't find Version named #{options[:version]}")
     end
+
     user_ids = options[:users]
 
-    scope = Issue.open.where("#{Issue.table_name}.assigned_to_id IS NOT NULL" +
-      " AND #{Project.table_name}.status = #{Project::STATUS_ACTIVE}" +
-      " AND #{Issue.table_name}.due_date <= ?", days.day.from_now.to_date
-    )
+    scope =
+      Issue.open.where(
+        "#{Issue.table_name}.assigned_to_id IS NOT NULL" \
+          " AND #{Project.table_name}.status = #{Project::STATUS_ACTIVE}" \
+          " AND #{Issue.table_name}.due_date <= ?", days.day.from_now.to_date
+      )
     scope = scope.where(:assigned_to_id => user_ids) if user_ids.present?
     scope = scope.where(:project_id => project.id) if project
     scope = scope.where(:fixed_version_id => target_version_id) if target_version_id.present?
@@ -605,23 +643,38 @@ class Mailer < ActionMailer::Base
   # Rake will likely end, causing the in-process thread pool to be deleted, before
   # any/all of the .deliver_later emails are processed
   def self.with_synched_deliveries(&block)
-    adapter = ActionMailer::DeliveryJob.queue_adapter
-    if adapter.is_a?(ActiveJob::QueueAdapters::AsyncAdapter)
-      ActionMailer::DeliveryJob.queue_adapter = ActiveJob::QueueAdapters::InlineAdapter.new
-    end
+    adapter = ActionMailer::MailDeliveryJob.queue_adapter
+    ActionMailer::MailDeliveryJob.queue_adapter = ActiveJob::QueueAdapters::InlineAdapter.new
     yield
   ensure
-    ActionMailer::DeliveryJob.queue_adapter = adapter
+    ActionMailer::MailDeliveryJob.queue_adapter = adapter
   end
 
   def mail(headers={}, &block)
+    # Add a display name to the From field if Setting.mail_from does not
+    # include it
+    begin
+      mail_from = Mail::Address.new(Setting.mail_from)
+      if mail_from.display_name.blank? && mail_from.comments.blank?
+        mail_from.display_name =
+          @author&.logged? ? @author.name : Setting.app_title
+      end
+      from = mail_from.format
+      list_id = "<#{mail_from.address.to_s.tr('@', '.')}>"
+    rescue Mail::Field::IncompleteParseError
+      # Use Setting.mail_from as it is if Mail::Address cannot parse it
+      # (probably the emission address is not RFC compliant)
+      from = Setting.mail_from.to_s
+      list_id = "<#{from.tr('@', '.')}>"
+    end
+
     headers.reverse_merge! 'X-Mailer' => 'Redmine',
             'X-Redmine-Host' => Setting.host_name,
             'X-Redmine-Site' => Setting.app_title,
             'X-Auto-Response-Suppress' => 'All',
             'Auto-Submitted' => 'auto-generated',
-            'From' => Setting.mail_from,
-            'List-Id' => "<#{Setting.mail_from.to_s.tr('@', '.')}>"
+            'From' => from,
+            'List-Id' => list_id
 
     # Replaces users with their email addresses
     [:to, :cc, :bcc].each do |key|
@@ -633,28 +686,21 @@ class Mailer < ActionMailer::Base
     # Removes the author from the recipients and cc
     # if the author does not want to receive notifications
     # about what the author do
-    if @author && @author.logged? && @author.pref.no_self_notified
+    if @author&.logged? && @author.pref.no_self_notified
       addresses = @author.mails
       headers[:to] -= addresses if headers[:to].is_a?(Array)
       headers[:cc] -= addresses if headers[:cc].is_a?(Array)
     end
 
-    if @author && @author.logged?
+    if @author&.logged?
       redmine_headers 'Sender' => @author.login
     end
 
-    # Blind carbon copy recipients
-    if Setting.bcc_recipients?
-      headers[:bcc] = [headers[:to], headers[:cc]].flatten.uniq.reject(&:blank?)
-      headers[:to] = nil
-      headers[:cc] = nil
-    end
-
     if @message_id_object
-      headers[:message_id] = "<#{self.class.message_id_for(@message_id_object)}>"
+      headers[:message_id] = "<#{self.class.message_id_for(@message_id_object, @user)}>"
     end
     if @references_objects
-      headers[:references] = @references_objects.collect {|o| "<#{self.class.references_for(o)}>"}.join(' ')
+      headers[:references] = @references_objects.collect {|o| "<#{self.class.references_for(o, @user)}>"}.join(' ')
     end
 
     if block_given?
@@ -669,11 +715,12 @@ class Mailer < ActionMailer::Base
 
   def self.deliver_mail(mail)
     return false if mail.to.blank? && mail.cc.blank? && mail.bcc.blank?
+
     begin
       # Log errors when raise_delivery_errors is set to false, Rails does not
       mail.raise_delivery_errors = true
       super
-    rescue Exception => e
+    rescue => e
       if ActionMailer::Base.raise_delivery_errors
         raise e
       else
@@ -705,33 +752,43 @@ class Mailer < ActionMailer::Base
 
   # Appends a Redmine header field (name is prepended with 'X-Redmine-')
   def redmine_headers(h)
-    h.each { |k,v| headers["X-Redmine-#{k}"] = v.to_s }
+    h.compact.each {|k, v| headers["X-Redmine-#{k}"] = v.to_s}
   end
 
-  def self.token_for(object, rand=true)
-    timestamp = object.send(object.respond_to?(:created_on) ? :created_on : :updated_on)
-    hash = [
-      "redmine",
-      "#{object.class.name.demodulize.underscore}-#{object.id}",
-      timestamp.strftime("%Y%m%d%H%M%S")
-    ]
-    if rand
-      hash << Redmine::Utils.random_hex(8)
+  def assignee_for_header(issue)
+    case issue.assigned_to
+    when User
+      issue.assigned_to.login
+    when Group
+      "Group (#{issue.assigned_to.name})"
     end
-    host = Setting.mail_from.to_s.strip.gsub(%r{^.*@|>}, '')
-    host = "#{::Socket.gethostname}.redmine" if host.empty?
-    "#{hash.join('.')}@#{host}"
   end
 
-  # Returns a Message-Id for the given object
-  def self.message_id_for(object)
-    token_for(object, true)
-  end
+  # Singleton class method is public
+  class << self
+    def token_for(object, user)
+      timestamp = object.send(object.respond_to?(:created_on) ? :created_on : :updated_on)
+      hash = [
+        "redmine",
+        "#{object.class.name.demodulize.underscore}-#{object.id}",
+        timestamp.utc.strftime("%Y%m%d%H%M%S")
+      ]
+      hash << user.id if user
+      host = Setting.mail_from.to_s.strip.gsub(%r{^.*@|>}, '')
+      host = "#{::Socket.gethostname}.redmine" if host.empty?
+      "#{hash.join('.')}@#{host}"
+    end
 
-  # Returns a uniq token for a given object referenced by all notifications
-  # related to this object
-  def self.references_for(object)
-    token_for(object, false)
+    # Returns a Message-Id for the given object
+    def message_id_for(object, user)
+      token_for(object, user)
+    end
+
+    # Returns a uniq token for a given object referenced by all notifications
+    # related to this object
+    def references_for(object, user)
+      token_for(object, user)
+    end
   end
 
   def message_id(object)
