@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,55 +20,56 @@
 require "digest/sha1"
 
 class User < Principal
+  include Redmine::Ciphering
   include Redmine::SafeAttributes
 
   # Different ways of displaying/sorting users
   USER_FORMATS = {
     :firstname_lastname => {
-        :string => '#{firstname} #{lastname}',
-        :order => %w(firstname lastname id),
-        :setting_order => 1
-      },
+      :string => '#{firstname} #{lastname}',
+      :order => %w(firstname lastname id),
+      :setting_order => 1
+    },
     :firstname_lastinitial => {
-        :string => '#{firstname} #{lastname.to_s.chars.first}.',
-        :order => %w(firstname lastname id),
-        :setting_order => 2
-      },
+      :string => '#{firstname} #{lastname.to_s.chars.first}.',
+      :order => %w(firstname lastname id),
+      :setting_order => 2
+    },
     :firstinitial_lastname => {
-        :string => '#{firstname.to_s.gsub(/(([[:alpha:]])[[:alpha:]]*\.?)/, \'\2.\')} #{lastname}',
-        :order => %w(firstname lastname id),
-        :setting_order => 2
-      },
+      :string => '#{firstname.to_s.gsub(/(([[:alpha:]])[[:alpha:]]*\.?)/, \'\2.\')} #{lastname}',
+      :order => %w(firstname lastname id),
+      :setting_order => 2
+    },
     :firstname => {
-        :string => '#{firstname}',
-        :order => %w(firstname id),
-        :setting_order => 3
-      },
+      :string => '#{firstname}',
+      :order => %w(firstname id),
+      :setting_order => 3
+    },
     :lastname_firstname => {
-        :string => '#{lastname} #{firstname}',
-        :order => %w(lastname firstname id),
-        :setting_order => 4
-      },
+      :string => '#{lastname} #{firstname}',
+      :order => %w(lastname firstname id),
+      :setting_order => 4
+    },
     :lastnamefirstname => {
-        :string => '#{lastname}#{firstname}',
-        :order => %w(lastname firstname id),
-        :setting_order => 5
-      },
+      :string => '#{lastname}#{firstname}',
+      :order => %w(lastname firstname id),
+      :setting_order => 5
+    },
     :lastname_comma_firstname => {
-        :string => '#{lastname}, #{firstname}',
-        :order => %w(lastname firstname id),
-        :setting_order => 6
-      },
+      :string => '#{lastname}, #{firstname}',
+      :order => %w(lastname firstname id),
+      :setting_order => 6
+    },
     :lastname => {
-        :string => '#{lastname}',
-        :order => %w(lastname id),
-        :setting_order => 7
-      },
+      :string => '#{lastname}',
+      :order => %w(lastname id),
+      :setting_order => 7
+    },
     :username => {
-        :string => '#{login}',
-        :order => %w(login id),
-        :setting_order => 8
-      },
+      :string => '#{login}',
+      :order => %w(login id),
+      :setting_order => 8
+    },
   }
 
   MAIL_NOTIFICATION_OPTIONS = [
@@ -84,14 +87,14 @@ class User < Principal
                           :after_remove => Proc.new {|user, group| group.user_removed(user)}
   has_many :changesets, :dependent => :nullify
   has_one :preference, :dependent => :destroy, :class_name => 'UserPreference'
-  has_one :rss_token, lambda {where "action='feeds'"}, :class_name => 'Token'
+  has_one :atom_token, lambda {where "action='feeds'"}, :class_name => 'Token'
   has_one :api_token, lambda {where "action='api'"}, :class_name => 'Token'
   has_one :email_address, lambda {where :is_default => true}, :autosave => true
   has_many :email_addresses, :dependent => :delete_all
   belongs_to :auth_source
 
-  scope :logged, lambda { where("#{User.table_name}.status <> #{STATUS_ANONYMOUS}") }
-  scope :status, lambda {|arg| where(arg.blank? ? nil : {:status => arg.to_i}) }
+  scope :logged, lambda {where("#{User.table_name}.status <> #{STATUS_ANONYMOUS}")}
+  scope :status, lambda {|arg| where(arg.blank? ? nil : {:status => arg.to_i})}
 
   acts_as_customizable
 
@@ -100,16 +103,18 @@ class User < Principal
   attr_accessor :remote_ip
 
   LOGIN_LENGTH_LIMIT = 60
-  MAIL_LENGTH_LIMIT = 60
+  MAIL_LENGTH_LIMIT = 254
 
-  validates_presence_of :login, :firstname, :lastname, :if => Proc.new { |user| !user.is_a?(AnonymousUser) }
-  validates_uniqueness_of :login, :if => Proc.new { |user| user.login_changed? && user.login.present? }, :case_sensitive => false
+  validates_presence_of :login, :firstname, :lastname, :if => Proc.new {|user| !user.is_a?(AnonymousUser)}
+  validates_uniqueness_of :login, :if => Proc.new {|user| user.login_changed? && user.login.present?}, :case_sensitive => false
   # Login must contain letters, numbers, underscores only
   validates_format_of :login, :with => /\A[a-z0-9_\-@\.]*\z/i
   validates_length_of :login, :maximum => LOGIN_LENGTH_LIMIT
   validates_length_of :firstname, :lastname, :maximum => 30
-  validates_length_of :identity_url, maximum: 255
   validates_inclusion_of :mail_notification, :in => MAIL_NOTIFICATION_OPTIONS.collect(&:first), :allow_blank => true
+  Setting::PASSWORD_CHAR_CLASSES.each do |k, v|
+    validates_format_of :password, :with => v, :message => :"must_contain_#{k}", :allow_blank => true, :if => Proc.new {Setting.password_required_char_classes.include?(k)}
+  end
   validate :validate_password_length
   validate do
     if password_confirmation && password != password_confirmation
@@ -126,27 +131,27 @@ class User < Principal
   after_save :update_notified_project_ids, :destroy_tokens, :deliver_security_notification
   after_destroy :deliver_security_notification
 
-  scope :admin, lambda {|*args|
+  scope :admin, (lambda do |*args|
     admin = args.size > 0 ? !!args.first : true
     where(:admin => admin)
-  }
-  scope :in_group, lambda {|group|
+  end)
+  scope :in_group, (lambda do |group|
     group_id = group.is_a?(Group) ? group.id : group.to_i
     where("#{User.table_name}.id IN (SELECT gu.user_id FROM #{table_name_prefix}groups_users#{table_name_suffix} gu WHERE gu.group_id = ?)", group_id)
-  }
-  scope :not_in_group, lambda {|group|
+  end)
+  scope :not_in_group, (lambda do |group|
     group_id = group.is_a?(Group) ? group.id : group.to_i
     where("#{User.table_name}.id NOT IN (SELECT gu.user_id FROM #{table_name_prefix}groups_users#{table_name_suffix} gu WHERE gu.group_id = ?)", group_id)
-  }
-  scope :sorted, lambda { order(*User.fields_for_order_statement)}
-  scope :having_mail, lambda {|arg|
+  end)
+  scope :sorted, lambda {order(*User.fields_for_order_statement)}
+  scope :having_mail, (lambda do |arg|
     addresses = Array.wrap(arg).map {|a| a.to_s.downcase}
     if addresses.any?
       joins(:email_addresses).where("LOWER(#{EmailAddress.table_name}.address) IN (?)", addresses).distinct
     else
       none
     end
-  }
+  end)
 
   def set_mail_notification
     self.mail_notification = Setting.default_notification_option if self.mail_notification.blank?
@@ -192,35 +197,24 @@ class User < Principal
     email_addresses.pluck(:address)
   end
 
-  def self.find_or_initialize_by_identity_url(url)
-    user = where(:identity_url => url).first
-    unless user
-      user = User.new
-      user.identity_url = url
-    end
-    user
-  end
-
-  def identity_url=(url)
-    if url.blank?
-      write_attribute(:identity_url, '')
-    else
-      begin
-        write_attribute(:identity_url, OpenIdAuthentication.normalize_identifier(url))
-      rescue OpenIdAuthentication::InvalidOpenId
-        # Invalid url, don't save
-      end
-    end
-    self.read_attribute(:identity_url)
+  # Returns the user that matches provided login and password, or nil
+  # AuthSource errors are caught, logged and nil is returned.
+  def self.try_to_login(login, password, active_only=true)
+    try_to_login!(login, password, active_only)
+  rescue AuthSourceException => e
+    logger.error "An error occured when authenticating #{login}: #{e.message}"
+    nil
   end
 
   # Returns the user that matches provided login and password, or nil
-  def self.try_to_login(login, password, active_only=true)
+  # AuthSource errors are passed through.
+  def self.try_to_login!(login, password, active_only=true)
     login = login.to_s.strip
     password = password.to_s
 
     # Make sure no one can sign in with an empty login or password
     return nil if login.empty? || password.empty?
+
     user = find_by_login(login)
     if user
       # user is already in local database
@@ -340,8 +334,7 @@ class User < Principal
 
   # Does the backend storage allow this user to change their password?
   def change_password_allowed?
-    return true if auth_source.nil?
-    return auth_source.allow_password_changes?
+    auth_source.nil? ? true : auth_source.allow_password_changes?
   end
 
   # Returns true if the user password has expired
@@ -365,15 +358,39 @@ class User < Principal
 
   # Generate and set a random password on given length
   def random_password(length=40)
-    chars = ("a".."z").to_a + ("A".."Z").to_a + ("0".."9").to_a
-    chars -= %w(0 O 1 l)
-    password = ''
-    length.times {|i| password << chars[SecureRandom.random_number(chars.size)] }
+    chars_list = [('A'..'Z').to_a, ('a'..'z').to_a, ('0'..'9').to_a]
+    # auto-generated passwords contain special characters only when admins
+    # require users to use passwords which contains special characters
+    if Setting.password_required_char_classes.include?('special_chars')
+      chars_list << ("\x20".."\x7e").to_a.select {|c| c =~ Setting::PASSWORD_CHAR_CLASSES['special_chars']}
+    end
+    chars_list.each {|v| v.reject! {|c| %(0O1l|'"`*).include?(c)}}
+
+    password = +''
+    chars_list.each do |chars|
+      password << chars[SecureRandom.random_number(chars.size)]
+      length -= 1
+    end
+    chars = chars_list.flatten
+    length.times {password << chars[SecureRandom.random_number(chars.size)]}
+    password = password.split('').shuffle(random: SecureRandom).join
     self.password = password
     self.password_confirmation = password
     self
   end
 
+  def twofa_active?
+    twofa_scheme.present?
+  end
+
+  def must_activate_twofa?
+    return false if twofa_active?
+
+    return true if Setting.twofa_required?
+    return true if Setting.twofa_required_for_administrators? && admin?
+    return true if Setting.twofa_optional? && groups.any?(&:twofa_required?)
+  end
+
   def pref
     self.preference ||= UserPreference.new(:user => self)
   end
@@ -398,12 +415,18 @@ class User < Principal
     self.pref[:comments_sorting] == 'desc'
   end
 
-  # Return user's RSS key (a 40 chars long string), used to access feeds
-  def rss_key
-    if rss_token.nil?
-      create_rss_token(:action => 'feeds')
+  # Return user's ATOM key (a 40 chars long string), used to access feeds
+  def atom_key
+    if atom_token.nil?
+      create_atom_token(:action => 'feeds')
     end
-    rss_token.value
+    atom_token.value
+  end
+
+  # TODO: remove in Redmine 6.0
+  def rss_key
+    ActiveSupport::Deprecation.warn "User.rss_key is deprecated and will be removed in Redmine 6.0. Please use User.atom_key instead."
+    atom_key
   end
 
   # Return user's API key (a 40 chars long string), used to access the API
@@ -434,6 +457,14 @@ class User < Principal
     Token.where(:user_id => id, :action => 'autologin', :value => value).delete_all
   end
 
+  def twofa_totp_key
+    read_ciphered_attribute(:twofa_totp_key)
+  end
+
+  def twofa_totp_key=(key)
+    write_ciphered_attribute(:twofa_totp_key, key)
+  end
+
   # Returns true if token is a valid session token for the user whose id is user_id
   def self.verify_session_token(user_id, token)
     return false if user_id.blank? || token.blank?
@@ -445,7 +476,14 @@ class User < Principal
     if Setting.session_timeout?
       scope = scope.where("updated_on > ?", Setting.session_timeout.to_i.minutes.ago)
     end
-    scope.update_all(:updated_on => Time.now) == 1
+    last_updated = scope.maximum(:updated_on)
+    if last_updated.nil?
+      false
+    elsif last_updated <= 1.minute.ago
+      scope.update_all(:updated_on => Time.now) == 1
+    else
+      true
+    end
   end
 
   # Return an array of project ids for which the user has explicitly turned mail notifications on
@@ -498,10 +536,16 @@ class User < Principal
     end
   end
 
-  def self.find_by_rss_key(key)
+  def self.find_by_atom_key(key)
     Token.find_active_user('feeds', key)
   end
 
+  # TODO: remove in Redmine 6.0
+  def self.find_by_rss_key(key)
+    ActiveSupport::Deprecation.warn "User.find_by_rss_key is deprecated and will be removed in Redmine 6.0. Please use User.find_by_atom_key instead."
+    self.find_by_atom_key(key)
+  end
+
   def self.find_by_api_key(key)
     Token.find_active_user('api', key)
   end
@@ -542,10 +586,14 @@ class User < Principal
 
   # Returns the day of +time+ according to user's time zone
   def time_to_date(time)
-    if time_zone.nil?
-      time.to_date
+    self.convert_time_to_user_timezone(time).to_date
+  end
+
+  def convert_time_to_user_timezone(time)
+    if self.time_zone
+      time.in_time_zone(self.time_zone)
     else
-      time.in_time_zone(time_zone).to_date
+      time.utc? ? time.localtime : time
     end
   end
 
@@ -562,14 +610,27 @@ class User < Principal
   def membership(project)
     project_id = project.is_a?(Project) ? project.id : project
 
-    @membership_by_project_id ||= Hash.new {|h, project_id|
-      h[project_id] = memberships.where(:project_id => project_id).first
-    }
+    @membership_by_project_id ||=
+      Hash.new do |h, project_id|
+        h[project_id] = memberships.where(:project_id => project_id).first
+      end
     @membership_by_project_id[project_id]
   end
 
   def roles
-    @roles ||= Role.joins(members: :project).where(["#{Project.table_name}.status <> ?", Project::STATUS_ARCHIVED]).where(Member.arel_table[:user_id].eq(id)).distinct
+    @roles ||=
+      Role.joins(members: :project).
+        where(["#{Project.table_name}.status <> ?", Project::STATUS_ARCHIVED]).
+          where(Member.arel_table[:user_id].eq(id)).distinct
+
+    if @roles.blank?
+      group_class = anonymous? ? GroupAnonymous : GroupNonMember
+      @roles = Role.joins(members: :project).
+        where(["#{Project.table_name}.status <> ? AND #{Project.table_name}.is_public = ?", Project::STATUS_ARCHIVED, true]).
+        where(Member.arel_table[:user_id].eq(group_class.first.id)).distinct
+    end
+
+    @roles
   end
 
   # Returns the user's bult-in role
@@ -581,6 +642,7 @@ class User < Principal
   def roles_for_project(project)
     # No role on archived projects
     return [] if project.nil? || project.archived?
+
     if membership = membership(project)
       membership.roles.to_a
     elsif project.is_public?
@@ -611,8 +673,8 @@ class User < Principal
     Project.unscoped do
       return @project_ids_by_role if @project_ids_by_role
 
-      group_class = anonymous? ? GroupAnonymous : GroupNonMember
-      group_id = group_class.pluck(:id).first
+      group_class = anonymous? ? GroupAnonymous.unscoped : GroupNonMember.unscoped
+      group_id = group_class.pick(:id)
 
       members = Member.joins(:project, :member_roles).
         where("#{Project.table_name}.status <> 9").
@@ -684,11 +746,12 @@ class User < Principal
 
       roles = roles_for_project(context)
       return false unless roles
-      roles.any? {|role|
+
+      roles.any? do |role|
         (context.is_public? || role.member?) &&
         role.allowed_to?(action) &&
         (block_given? ? yield(role, self) : true)
-      }
+      end
     elsif context && context.is_a?(Array)
       if context.empty?
         false
@@ -704,10 +767,10 @@ class User < Principal
 
       # authorize if user has at least one role that has this permission
       roles = self.roles.to_a | [builtin_role]
-      roles.any? {|role|
+      roles.any? do |role|
         role.allowed_to?(action) &&
         (block_given? ? yield(role, self) : true)
-      }
+      end
     else
       false
     end
@@ -735,29 +798,29 @@ class User < Principal
       (!admin? || User.active.admin.where("id <> ?", id).exists?)
   end
 
-  safe_attributes 'firstname',
+  safe_attributes(
+    'firstname',
     'lastname',
     'mail',
     'mail_notification',
     'notified_project_ids',
     'language',
     'custom_field_values',
-    'custom_fields',
-    'identity_url'
-
-  safe_attributes 'login',
-    :if => lambda {|user, current_user| user.new_record?}
-
-  safe_attributes 'status',
+    'custom_fields')
+  safe_attributes(
+    'login',
+    :if => lambda {|user, current_user| user.new_record?})
+  safe_attributes(
+    'status',
     'auth_source_id',
     'generate_password',
     'must_change_passwd',
     'login',
     'admin',
-    :if => lambda {|user, current_user| current_user.admin?}
-
-  safe_attributes 'group_ids',
-    :if => lambda {|user, current_user| current_user.admin? && !user.new_record?}
+    :if => lambda {|user, current_user| current_user.admin?})
+  safe_attributes(
+    'group_ids',
+    :if => lambda {|user, current_user| current_user.admin? && !user.new_record?})
 
   # Utility method to help check if a user should be notified about an
   # event.
@@ -787,6 +850,10 @@ class User < Principal
     end
   end
 
+  def notify_about_high_priority_issues?
+    self.pref.notify_about_high_priority_issues
+  end
+
   def self.current=(user)
     RequestStore.store[:current_user] = user
   end
@@ -813,6 +880,7 @@ class User < Principal
     transaction do
       User.where("salt IS NULL OR salt = ''").find_each do |user|
         next if user.hashed_password.blank?
+
         salt = User.generate_salt
         hashed_password = User.hash_password("#{salt}#{user.hashed_password}")
         User.where(:id => user.id).update_all(:salt => salt, :hashed_password => hashed_password)
@@ -820,10 +888,22 @@ class User < Principal
     end
   end
 
+  def bookmarked_project_ids
+    project_ids = []
+    bookmarked_project_ids = self.pref[:bookmarked_project_ids]
+    project_ids = bookmarked_project_ids.split(',') unless bookmarked_project_ids.nil?
+    project_ids.map(&:to_i)
+  end
+
+  def self.prune(age)
+    User.where("created_on < ? AND status = ?", Time.now - age, STATUS_REGISTERED).destroy_all
+  end
+
   protected
 
   def validate_password_length
     return if password.blank? && generate_password?
+
     # Password length validation based on setting
     if !password.nil? && password.size < Setting.password_min_length.to_i
       errors.add(:password, :too_short, :count => Setting.password_min_length.to_i)
@@ -848,7 +928,7 @@ class User < Principal
   # This helps to keep the account secure in case the associated email account
   # was compromised.
   def destroy_tokens
-    if saved_change_to_hashed_password? || (saved_change_to_status? && !active?)
+    if saved_change_to_hashed_password? || (saved_change_to_status? && !active?) || (saved_change_to_twofa_scheme? && twofa_scheme.present?)
       tokens = ['recovery', 'autologin', 'session']
       Token.where(:user_id => id, :action => tokens).delete_all
     end
@@ -880,17 +960,24 @@ class User < Principal
     Token.where('user_id = ?', id).delete_all
     Watcher.where('user_id = ?', id).delete_all
     WikiContent.where(['author_id = ?', id]).update_all(['author_id = ?', substitute.id])
-    WikiContent::Version.where(['author_id = ?', id]).update_all(['author_id = ?', substitute.id])
+    WikiContentVersion.where(['author_id = ?', id]).update_all(['author_id = ?', substitute.id])
+    user_custom_field_ids = CustomField.where(field_format: 'user').ids
+    if user_custom_field_ids.any?
+      CustomValue.where(custom_field_id: user_custom_field_ids, value: self.id.to_s).delete_all
+    end
   end
 
-  # Return password digest
-  def self.hash_password(clear_password)
-    Digest::SHA1.hexdigest(clear_password || "")
-  end
+  # Singleton class method is public
+  class << self
+    # Return password digest
+    def hash_password(clear_password)
+      Digest::SHA1.hexdigest(clear_password || "")
+    end
 
-  # Returns a 128bits random salt as a hex string (32 chars long)
-  def self.generate_salt
-    Redmine::Utils.random_hex(16)
+    # Returns a 128bits random salt as a hex string (32 chars long)
+    def generate_salt
+      Redmine::Utils.random_hex(16)
+    end
   end
 
   # Send a security notification to all admins if the user has gained/lost admin privileges
@@ -906,16 +993,13 @@ class User < Principal
     if (admin? && saved_change_to_id? && active?) ||    # newly created admin
        (admin? && saved_change_to_admin? && active?) || # regular user became admin
        (admin? && saved_change_to_status? && active?)   # locked admin became active again
-
-       deliver = true
-       options[:message] = :mail_body_security_notification_add
-
+      deliver = true
+      options[:message] = :mail_body_security_notification_add
     elsif (admin? && destroyed? && active?) ||      # active admin user was deleted
           (!admin? && saved_change_to_admin? && active?) || # admin is no longer admin
           (admin? && saved_change_to_status? && !active?)   # admin was locked
-
-          deliver = true
-          options[:message] = :mail_body_security_notification_remove
+      deliver = true
+      options[:message] = :mail_body_security_notification_remove
     end
 
     if deliver
@@ -924,54 +1008,3 @@ class User < Principal
     end
   end
 end
-
-class AnonymousUser < User
-  validate :validate_anonymous_uniqueness, :on => :create
-
-  self.valid_statuses = [STATUS_ANONYMOUS]
-
-  def validate_anonymous_uniqueness
-    # There should be only one AnonymousUser in the database
-    errors.add :base, 'An anonymous user already exists.' if AnonymousUser.exists?
-  end
-
-  def available_custom_fields
-    []
-  end
-
-  # Overrides a few properties
-  def logged?; false end
-  def admin; false end
-  def name(*args); I18n.t(:label_user_anonymous) end
-  def mail=(*args); nil end
-  def mail; nil end
-  def time_zone; nil end
-  def rss_key; nil end
-
-  def pref
-    UserPreference.new(:user => self)
-  end
-
-  # Returns the user's bult-in role
-  def builtin_role
-    @builtin_role ||= Role.anonymous
-  end
-
-  def membership(*args)
-    nil
-  end
-
-  def member_of?(*args)
-    false
-  end
-
-  # Anonymous user can not be destroyed
-  def destroy
-    false
-  end
-
-  protected
-
-  def instantiate_email_address
-  end
-end
