@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,14 +19,16 @@
 
 module Redmine
   module AccessKeys
-    ACCESSKEYS = {:edit => 'e',
-                  :preview => 'r',
-                  :quick_search => 'f',
-                  :search => '4',
-                  :new_issue => '7',
-                  :previous => 'p',
-                  :next => 'n'
-                 }.freeze unless const_defined?(:ACCESSKEYS)
+    unless const_defined?(:ACCESSKEYS)
+      ACCESSKEYS = {
+        :edit => 'e',
+        :quick_search => 'f',
+        :search => '4',
+        :new_issue => '7',
+        :previous => 'p',
+        :next => 'n'
+      }.freeze
+    end
 
     def self.key_for(action)
       ACCESSKEYS[action]
