@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,8 +20,11 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class MyControllerTest < Redmine::ControllerTest
-  fixtures :users, :email_addresses, :user_preferences, :roles, :projects, :members, :member_roles,
-  :issues, :issue_statuses, :trackers, :enumerations, :custom_fields, :auth_sources, :queries, :enabled_modules
+  fixtures :users, :email_addresses, :user_preferences,
+           :roles, :projects, :members, :member_roles,
+           :issues, :issue_statuses, :trackers, :enumerations,
+           :custom_fields, :auth_sources, :queries, :enabled_modules,
+           :journals, :projects_trackers
 
   def setup
     @request.session[:user_id] = 2
@@ -41,17 +46,24 @@ class MyControllerTest < Redmine::ControllerTest
     preferences = User.find(2).pref
     preferences[:my_page_layout] = {'top' => ['timelog']}
     preferences.save!
-    with_issue =    TimeEntry.create!(:user => User.find(2), :spent_on => Date.yesterday, :hours => 2.5, :activity_id => 10, :issue_id => 1)
-    without_issue = TimeEntry.create!(:user => User.find(2), :spent_on => Date.yesterday, :hours => 3.5, :activity_id => 10, :project_id => 1)
-
+    with_issue =
+      TimeEntry.create!(
+        :user => User.find(2), :spent_on => Date.yesterday,
+        :hours => 2.5, :activity_id => 10, :issue_id => 1
+      )
+    without_issue =
+      TimeEntry.create!(
+        :user => User.find(2), :spent_on => Date.yesterday,
+        :hours => 3.5, :activity_id => 10, :project_id => 1
+      )
     get :page
     assert_response :success
     assert_select "tr#time-entry-#{with_issue.id}" do
       assert_select 'td.subject a[href="/issues/1"]'
-      assert_select 'td.hours', :text => '2.50'
+      assert_select 'td.hours', :text => '2:30'
     end
     assert_select "tr#time-entry-#{without_issue.id}" do
-      assert_select 'td.hours', :text => '3.50'
+      assert_select 'td.hours', :text => '3:30'
     end
   end
 
@@ -75,7 +87,8 @@ class MyControllerTest < Redmine::ControllerTest
   def test_page_with_assigned_issues_block_and_custom_columns
     preferences = User.find(2).pref
     preferences.my_page_layout = {'top' => ['issuesassignedtome']}
-    preferences.my_page_settings = {'issuesassignedtome' => {:columns => ['tracker', 'subject', 'due_date']}}
+    preferences.my_page_settings =
+      {'issuesassignedtome' => {:columns => ['tracker', 'subject', 'due_date']}}
     preferences.save!
 
     get :page
@@ -114,7 +127,11 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_page_with_issuequery_block_and_global_query
     user = User.find(2)
-    query = IssueQuery.create!(:name => 'All issues', :user => user, :column_names => [:tracker, :subject, :status, :assigned_to])
+    query =
+      IssueQuery.create!(
+        :name => 'All issues', :user => user,
+        :column_names => [:tracker, :subject, :status, :assigned_to]
+      )
     user.pref.my_page_layout = {'top' => ['issuequery']}
     user.pref.my_page_settings = {'issuequery' => {:query_id => query.id}}
     user.pref.save!
@@ -134,7 +151,12 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_page_with_issuequery_block_and_project_query
     user = User.find(2)
-    query = IssueQuery.create!(:name => 'All issues', :project => Project.find(1), :user => user, :column_names => [:tracker, :subject, :status, :assigned_to])
+    query =
+      IssueQuery.create!(
+        :name => 'All issues', :project => Project.find(1),
+        :user => user,
+        :column_names => [:tracker, :subject, :status, :assigned_to]
+      )
     user.pref.my_page_layout = {'top' => ['issuequery']}
     user.pref.my_page_settings = {'issuequery' => {:query_id => query.id}}
     user.pref.save!
@@ -154,9 +176,15 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_page_with_issuequery_block_and_query_should_display_custom_columns
     user = User.find(2)
-    query = IssueQuery.create!(:name => 'All issues', :user => user, :column_names => [:tracker, :subject, :status, :assigned_to])
+    query =
+      IssueQuery.create!(
+        :name => 'All issues', :user => user,
+        :column_names => [:tracker, :subject, :status, :assigned_to]
+      )
     user.pref.my_page_layout = {'top' => ['issuequery']}
-    user.pref.my_page_settings = {'issuequery' => {:query_id => query.id, :columns => [:subject, :due_date]}}
+    user.pref.my_page_settings = {
+      'issuequery' => {:query_id => query.id, :columns => [:subject, :due_date]}
+    }
     user.pref.save!
 
     get :page
@@ -171,13 +199,17 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_page_with_multiple_issuequery_blocks
     user = User.find(2)
-    query1 = IssueQuery.create!(:name => 'All issues', :user => user, :column_names => [:tracker, :subject, :status, :assigned_to])
-    query2 = IssueQuery.create!(:name => 'Other issues', :user => user, :column_names => [:tracker, :subject, :priority])
+    query1 =
+      IssueQuery.create!(:name => 'All issues', :user => user,
+                         :column_names => [:tracker, :subject, :status, :assigned_to])
+    query2 =
+      IssueQuery.create!(:name => 'Other issues', :user => user,
+                         :column_names => [:tracker, :subject, :priority])
     user.pref.my_page_layout = {'top' => ['issuequery__1', 'issuequery']}
     user.pref.my_page_settings = {
-        'issuequery' => {:query_id => query1.id, :columns => [:subject, :due_date]},
-        'issuequery__1' => {:query_id => query2.id}
-      }
+      'issuequery' => {:query_id => query1.id, :columns => [:subject, :due_date]},
+      'issuequery__1' => {:query_id => query2.id}
+    }
     user.pref.save!
 
     get :page
@@ -201,6 +233,7 @@ class MyControllerTest < Redmine::ControllerTest
   def test_page_with_activity
     user = User.find(2)
     user.pref.my_page_layout = {'top' => ['activity']}
+    user.pref.time_zone = 'UTC'
     user.pref.save!
 
     get :page
@@ -208,7 +241,10 @@ class MyControllerTest < Redmine::ControllerTest
 
     assert_select 'div#block-activity' do
       assert_select 'h3' do
-        assert_select 'a[href=?]', activity_path(from: Date.current, user_id: user.id),  :text => 'Activity'
+        assert_select(
+          'a[href=?]', activity_path(from: User.current.today, user_id: user.id),
+          :text => 'Activity'
+        )
       end
       assert_select 'div#activity' do
         assert_select 'dt', 10
@@ -216,6 +252,31 @@ class MyControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_page_with_updated_issues_block
+    preferences = User.find(2).pref
+    preferences.my_page_layout = {'top' => ['issuesupdatedbyme']}
+    preferences.my_page_settings = {'issuesupdatedbyme' => {}}
+    preferences.save!
+
+    project = Project.find(3)
+    project.close
+
+    get :page
+
+    assert_response :success
+    assert_select '#block-issuesupdatedbyme' do
+      report_url = CGI.unescape(css_select('h3 a').first.attr(:href))
+      assert_match 'f[]=project.status', report_url
+      assert_match 'v[project.status][]=1', report_url
+      assert_match 'f[]=updated_by', report_url
+      assert_match 'v[updated_by][]=me', report_url
+
+      assert_select 'table.issues tbody tr', 2
+      assert_select 'table.issues tbody tr[id=?]', 'issue-1', 1, :title => 'Cannot print recipes'
+      assert_select 'table.issues tbody tr[id=?]', 'issue-14', 0
+    end
+  end
+
   def test_page_with_all_blocks
     blocks = Redmine::MyPage.blocks.keys
     preferences = User.find(2).pref
@@ -245,7 +306,7 @@ class MyControllerTest < Redmine::ControllerTest
 
     assert_response :success
     assert_select '#block-issuesassignedtome table.issues tbody' do
-      report_url = css_select('h3 a').map {|e| e.attr('href')}.first
+      report_url = css_select('h3 a').map {|e| e.attr(:href)}.first
       assert_match 'f%5B%5D=project.status', report_url
       assert_match 'v%5Bproject.status%5D%5B%5D=1', report_url
 
@@ -273,7 +334,7 @@ class MyControllerTest < Redmine::ControllerTest
 
     assert_response :success
     assert_select '#block-issuesreportedbyme' do
-      report_url = css_select('h3 a').map {|e| e.attr('href')}.first
+      report_url = css_select('h3 a').map {|e| e.attr(:href)}.first
       assert_match 'f%5B%5D=project.status', report_url
       assert_match 'v%5Bproject.status%5D%5B%5D=1', report_url
 
@@ -305,7 +366,7 @@ class MyControllerTest < Redmine::ControllerTest
 
     assert_response :success
     assert_select '#block-issueswatched table.issues tbody' do
-      report_url = css_select('h3 a').map {|e| e.attr('href')}.first
+      report_url = css_select('h3 a').map {|e| e.attr(:href)}.first
       assert_match 'f%5B%5D=project.status', report_url
       assert_match 'v%5Bproject.status%5D%5B%5D=1', report_url
 
@@ -337,9 +398,11 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_my_account_with_avatar_enabled_should_link_to_edit_avatar
     with_settings :gravatar_enabled => '1' do
-      get :account
-      assert_response :success
-      assert_select 'a[href=?] img.gravatar', 'https://gravatar.com'
+      Redmine::Configuration.with 'avatar_server_url' => 'https://gravatar.com' do
+        get :account
+        assert_response :success
+        assert_select 'a[href=?] img.gravatar', 'https://gravatar.com'
+      end
     end
   end
 
@@ -351,8 +414,55 @@ class MyControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_page_with_calendar
+    date = '2020-10-21'
+    subject = 'calendar on my page'
+    issue = Issue.generate!(:start_date => date,
+                            :due_date   => date,
+                            :project_id => 1,
+                            :tracker_id => 1,
+                            :subject => subject)
+
+    travel_to date
+
+    preferences = User.find(2).pref
+    preferences[:my_page_layout] = {'top' => ['calendar']}
+    preferences.save!
+
+    with_settings :start_of_week => 7 do
+      get :page
+    end
+    assert_response :success
+
+    assert_select 'form[data-cm-url=?]', '/issues/context_menu'
+
+    assert_select 'table.cal' do
+      assert_select 'tr' do
+        assert_select 'td' do
+          assert_select(
+            'div.issue.hascontextmenu.tooltip.starting.ending',
+            :text => /eCookbook.*#{subject}/m
+          ) do
+            assert_select(
+              'a.issue[href=?]', "/issues/#{issue.id}",
+              :text => "Bug ##{issue.id}"
+            )
+            assert_select(
+              'input[name=?][type=?][value=?]',
+              'ids[]',
+              'checkbox',
+              issue.id.to_s
+            )
+          end
+        end
+      end
+    end
+  end
+
   def test_update_account
-    post :account, :params => {
+    put(
+      :account,
+      :params => {
         :user => {
           :firstname => "Joe",
           :login => "root",
@@ -361,10 +471,9 @@ class MyControllerTest < Redmine::ControllerTest
           :custom_field_values => {
             "4" => "0100562500"
           }
-
         }
       }
-
+    )
     assert_redirected_to '/my/account'
     user = User.find(2)
     assert_equal "Joe", user.firstname
@@ -377,22 +486,67 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_update_account_should_send_security_notification
     ActionMailer::Base.deliveries.clear
-    post :account, :params => {
+    put(
+      :account,
+      :params => {
         :user => {
           :mail => 'foobar@example.com'
 
         }
       }
-
+    )
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
     assert_mail_body_match '0.0.0.0', mail
-    assert_mail_body_match I18n.t(:mail_body_security_notification_change_to, field: I18n.t(:field_mail), value: 'foobar@example.com'), mail
+    assert_mail_body_match(
+      I18n.t(:mail_body_security_notification_change_to,
+             :field => I18n.t(:field_mail), :value => 'foobar@example.com'),
+      mail
+    )
     assert_select_email do
       assert_select 'a[href^=?]', 'http://localhost:3000/my/account', :text => 'My account'
     end
     # The old email address should be notified about the change for security purposes
-    assert [mail.bcc, mail.cc].flatten.include?(User.find(2).mail)
-    assert [mail.bcc, mail.cc].flatten.include?('foobar@example.com')
+    assert mail.to.include?(User.find(2).mail)
+    assert mail.to.include?('foobar@example.com')
+  end
+
+  def test_my_account_notify_about_high_priority_issues_preference
+    # normally, preference should be shown
+    get :account
+    assert_select 'label[for="pref_notify_about_high_priority_issues"]'
+
+    # preference should be persisted
+    put(
+      :account,
+      :params => {
+        :pref => {
+          notify_about_high_priority_issues: '1'
+        }
+      }
+    )
+    assert User.find(2).notify_about_high_priority_issues?
+
+    # preference should be hidden if there aren't any priorities
+    Issue.destroy_all
+    IssuePriority.destroy_all
+    get :account
+    assert_select 'label[for="pref_notify_about_high_priority_issues"]', false
+
+    # preference should be hidden if there isn't a "high" priority
+    a = IssuePriority.create! name: 'A'
+    get :account
+    assert_select 'label[for="pref_notify_about_high_priority_issues"]', false
+
+    # preference should be shown if there are at least two priorities (one low, one high)
+    b = IssuePriority.create! name: 'B'
+    get :account
+    assert_select 'label[for="pref_notify_about_high_priority_issues"]'
+
+    # preference should be hidden if the highest priority is the default one,
+    # because that means that there is no "high" priority
+    b.update! is_default: true
+    get :account
+    assert_select 'label[for="pref_notify_about_high_priority_issues"]', false
   end
 
   def test_my_account_should_show_destroy_link
@@ -417,9 +571,12 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_post_destroy_without_confirmation_should_destroy_account
     assert_difference 'User.count', -1 do
-      post :destroy, :params => {
+      post(
+        :destroy,
+        :params => {
           :confirm => '1'
         }
+      )
     end
     assert_redirected_to '/'
     assert_match /deleted/i, flash[:notice]
@@ -429,9 +586,12 @@ class MyControllerTest < Redmine::ControllerTest
     User.any_instance.stubs(:own_account_deletable?).returns(false)
 
     assert_no_difference 'User.count' do
-      post :destroy, :params => {
+      post(
+        :destroy,
+        :params => {
           :confirm => '1'
         }
+      )
     end
     assert_redirected_to '/my/account'
   end
@@ -445,21 +605,27 @@ class MyControllerTest < Redmine::ControllerTest
   end
 
   def test_update_password
-    post :password, :params => {
+    post(
+      :password,
+      :params => {
         :password => 'jsmith',
         :new_password => 'secret123',
         :new_password_confirmation => 'secret123'
       }
+    )
     assert_redirected_to '/my/account'
     assert User.try_to_login('jsmith', 'secret123')
   end
 
   def test_update_password_with_non_matching_confirmation
-    post :password, :params => {
+    post(
+      :password,
+      :params => {
         :password => 'jsmith',
         :new_password => 'secret123',
         :new_password_confirmation => 'secret1234'
       }
+    )
     assert_response :success
     assert_select_error /Password doesn.*t match confirmation/
     assert User.try_to_login('jsmith', 'jsmith')
@@ -467,11 +633,14 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_update_password_with_wrong_password
     # wrong password
-    post :password, :params => {
+    post(
+      :password,
+      :params => {
         :password => 'wrongpassword',
         :new_password => 'secret123',
         :new_password_confirmation => 'secret123'
       }
+    )
     assert_response :success
     assert_equal 'Wrong password', flash[:error]
     assert User.try_to_login('jsmith', 'jsmith')
@@ -487,12 +656,14 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_update_password_should_send_security_notification
     ActionMailer::Base.deliveries.clear
-    post :password, :params => {
+    post(
+      :password,
+      :params => {
         :password => 'jsmith',
         :new_password => 'secret123',
         :new_password_confirmation => 'secret123'
       }
-
+    )
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
     assert_mail_body_no_match 'secret123', mail # just to be sure: pw should never be sent!
     assert_select_email do
@@ -504,57 +675,77 @@ class MyControllerTest < Redmine::ControllerTest
     user = User.generate!(:language => 'en')
     @request.session[:user_id] = user.id
 
-    post :update_page, :params => {
+    post(
+      :update_page,
+      :params => {
         :settings => {
           'issuesassignedtome' => {
-          'columns' => ['subject', 'due_date']}
+            'columns' => ['subject', 'due_date']
+          }
         }
       },
       :xhr => true
+    )
     assert_response :success
     assert_include '$("#block-issuesassignedtome").replaceWith(', response.body
     assert_include 'Due date', response.body
 
-    assert_equal({:columns => ['subject', 'due_date']}, user.reload.pref.my_page_settings('issuesassignedtome'))
+    assert_equal({:columns => ['subject', 'due_date']},
+                 user.reload.pref.my_page_settings('issuesassignedtome'))
   end
 
   def test_add_block
-    post :add_block, :params => {
+    post(
+      :add_block,
+      :params => {
         :block => 'issueswatched'
       }
+    )
     assert_redirected_to '/my/page'
     assert User.find(2).pref[:my_page_layout]['top'].include?('issueswatched')
   end
 
   def test_add_block_xhr
-    post :add_block, :params => {
+    post(
+      :add_block,
+      :params => {
         :block => 'issueswatched'
       },
       :xhr => true
+    )
     assert_response :success
     assert_include 'issueswatched', User.find(2).pref[:my_page_layout]['top']
   end
 
   def test_add_invalid_block_should_error
-    post :add_block, :params => {
+    post(
+      :add_block,
+      :params => {
         :block => 'invalid'
       }
+    )
     assert_response 422
   end
 
   def test_remove_block
-    post :remove_block, :params => {
+    post(
+      :remove_block,
+      :params => {
         :block => 'issuesassignedtome'
       }
+    )
     assert_redirected_to '/my/page'
     assert !User.find(2).pref[:my_page_layout].values.flatten.include?('issuesassignedtome')
   end
 
   def test_remove_block_xhr
-    post :remove_block, :params => {
+    post(
+      :remove_block,
+      :params => {
         :block => 'issuesassignedtome'
       },
       :xhr => true
+    )
     assert_response :success
     assert_include '$("#block-issuesassignedtome").remove();', response.body
     assert !User.find(2).pref[:my_page_layout].values.flatten.include?('issuesassignedtome')
@@ -562,48 +753,56 @@ class MyControllerTest < Redmine::ControllerTest
 
   def test_order_blocks
     pref = User.find(2).pref
-    pref.my_page_layout = {'left' => ['news', 'calendar','documents']}
+    pref.my_page_layout = {'left' => ['news', 'calendar', 'documents']}
     pref.save!
 
-    post :order_blocks, :params => {
+    post(
+      :order_blocks,
+      :params => {
         :group => 'left',
         :blocks => ['documents', 'calendar', 'news']
       },
       :xhr => true
+    )
     assert_response :success
     assert_equal ['documents', 'calendar', 'news'], User.find(2).pref.my_page_layout['left']
   end
 
   def test_move_block
     pref = User.find(2).pref
-    pref.my_page_layout = {'left' => ['news','documents'], 'right' => ['calendar']}
+    pref.my_page_layout = {'left' => ['news', 'documents'], 'right' => ['calendar']}
     pref.save!
 
-    post :order_blocks, :params => {
+    post(
+      :order_blocks,
+      :params => {
         :group => 'left',
         :blocks => ['news', 'calendar', 'documents']
       },
       :xhr => true
+    )
     assert_response :success
-    assert_equal({'left' => ['news', 'calendar', 'documents'], 'right' => []}, User.find(2).pref.my_page_layout)
+    assert_equal({'left' => ['news', 'calendar', 'documents'],
+                 'right' => []},
+                 User.find(2).pref.my_page_layout)
   end
 
-  def test_reset_rss_key_with_existing_key
-    @previous_token_value = User.find(2).rss_key # Will generate one if it's missing
-    post :reset_rss_key
+  def test_reset_atom_key_with_existing_key
+    @previous_token_value = User.find(2).atom_key # Will generate one if it's missing
+    post :reset_atom_key
 
-    assert_not_equal @previous_token_value, User.find(2).rss_key
-    assert User.find(2).rss_token
+    assert_not_equal @previous_token_value, User.find(2).atom_key
+    assert User.find(2).atom_token
     assert_match /reset/, flash[:notice]
     assert_redirected_to '/my/account'
   end
 
-  def test_reset_rss_key_without_existing_key
+  def test_reset_atom_key_without_existing_key
     Token.delete_all
-    assert_nil User.find(2).rss_token
-    post :reset_rss_key
+    assert_nil User.find(2).atom_token
+    post :reset_atom_key
 
-    assert User.find(2).rss_token
+    assert User.find(2).atom_token
     assert_match /reset/, flash[:notice]
     assert_redirected_to '/my/account'
   end
