@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,7 +22,7 @@ require File.expand_path('../../test_helper', __FILE__)
 class UsersControllerTest < Redmine::ControllerTest
   include Redmine::I18n
 
-  fixtures :users, :email_addresses, :projects, :members, :member_roles, :roles,
+  fixtures :users, :user_preferences, :email_addresses, :projects, :members, :member_roles, :roles,
            :custom_fields, :custom_values, :groups_users,
            :auth_sources,
            :enabled_modules,
@@ -64,27 +66,100 @@ class UsersControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_index_should_not_show_2fa_filter_and_column_if_disabled
+    with_settings twofa: "0" do
+      get :index
+      assert_response :success
+
+      assert_select "select#twofa", 0
+      assert_select 'td.twofa', 0
+    end
+  end
+
+  def test_index_filter_by_twofa_yes
+    with_settings twofa: "1" do
+      user = User.find(1)
+      user.twofa_totp_key = "AVYA3RARZ3GY3VWT7MIEJ72I5TTJRO3X"
+      user.twofa_scheme = "totp"
+      user.save
+
+      get :index, :params => {:twofa => '1'}
+      assert_response :success
+
+      assert_select "select#twofa", 1
+
+      assert_select 'tr.user', 1
+      assert_select 'td.twofa.tick .icon-checked'
+    end
+  end
+
+  def test_index_filter_by_twofa_no
+    with_settings twofa: "1" do
+      user = User.find(1)
+      user.twofa_totp_key = "AVYA3RARZ3GY3VWT7MIEJ72I5TTJRO3X"
+      user.twofa_scheme = "totp"
+      user.save
+
+      get :index, :params => {:twofa => '0'}
+      assert_response :success
+
+      assert_select "select#twofa", 1
+      assert_select "td.twofa.tick" do
+        assert_select "span.icon-checked", 0
+      end
+    end
+  end
+
   def test_index_csv
     with_settings :default_language => 'en' do
-      get :index, :params => { :format => 'csv' }
+      user = User.logged.status(1).first
+      user.update(passwd_changed_on: Time.current.last_month, twofa_scheme: 'totp')
+      get :index, params: {format: 'csv'}
       assert_response :success
 
       assert_equal User.logged.status(1).count, response.body.chomp.split("\n").size - 1
-      assert_include 'active', response.body
-      assert_not_include 'locked', response.body
-      assert_equal 'text/csv', @response.media_type
+      assert_include format_time(user.updated_on), response.body.split("\n").second
+      assert_include format_time(user.passwd_changed_on), response.body.split("\n").second
+
+      # status
+      assert_include 'active', response.body.split("\n").second
+      assert_not_include 'locked', response.body.split("\n").second
+
+      # twofa_scheme
+      assert_include 'Authenticator app', response.body.split("\n").second
+      assert_include 'disabled', response.body.split("\n").third
+
+      assert_equal 'text/csv; header=present', @response.media_type
+    end
+  end
+
+  def test_index_csv_with_custom_field_columns
+    float_custom_field = UserCustomField.generate!(:name => 'float field', :field_format => 'float')
+    date_custom_field = UserCustomField.generate!(:name => 'date field', :field_format => 'date')
+    user = User.last
+    user.custom_field_values = {float_custom_field.id.to_s => 2.1, date_custom_field.id.to_s => '2020-01-10'}
+    user.save
+
+    User.find(@request.session[:user_id]).update(:language => nil)
+    with_settings :default_language => 'fr' do
+      get :index, :params => {:name => user.lastname, :format => 'csv'}
+      assert_response :success
+
+      assert_include 'float field;date field', response.body
+      assert_include '2,10;10/01/2020', response.body
+      assert_equal 'text/csv; header=present', @response.media_type
     end
   end
 
   def test_index_csv_with_status_filter
     with_settings :default_language => 'en' do
-      get :index, :params => { :status => 3, :format => 'csv' }
+      get :index, :params => {:status => 3, :format => 'csv'}
       assert_response :success
 
       assert_equal User.logged.status(3).count, response.body.chomp.split("\n").size - 1
       assert_include 'locked', response.body
       assert_not_include 'active', response.body
-      assert_equal 'text/csv', @response.media_type
+      assert_equal 'text/csv; header=present', @response.media_type
     end
   end
 
@@ -94,7 +169,7 @@ class UsersControllerTest < Redmine::ControllerTest
 
     assert_equal User.logged.like('John').count, response.body.chomp.split("\n").size - 1
     assert_include 'John', response.body
-    assert_equal 'text/csv', @response.media_type
+    assert_equal 'text/csv; header=present', @response.media_type
   end
 
   def test_index_csv_with_group_filter
@@ -102,7 +177,7 @@ class UsersControllerTest < Redmine::ControllerTest
     assert_response :success
 
     assert_equal Group.find(10).users.count, response.body.chomp.split("\n").size - 1
-    assert_equal 'text/csv', @response.media_type
+    assert_equal 'text/csv; header=present', @response.media_type
   end
 
   def test_show
@@ -110,6 +185,9 @@ class UsersControllerTest < Redmine::ControllerTest
     get :show, :params => {:id => 2}
     assert_response :success
     assert_select 'h2', :text => /John Smith/
+
+    # groups block should not be rendeder for users which are not part of any group
+    assert_select 'div#groups', 0
   end
 
   def test_show_should_display_visible_custom_fields
@@ -118,7 +196,7 @@ class UsersControllerTest < Redmine::ControllerTest
     get :show, :params => {:id => 2}
     assert_response :success
 
-    assert_select 'li[class=?]', 'cf_4', :text => /Phone number/
+    assert_select 'li.cf_4.string_cf', :text => /Phone number/
   end
 
   def test_show_should_not_display_hidden_custom_fields
@@ -168,8 +246,20 @@ class UsersControllerTest < Redmine::ControllerTest
     get :show, :params => {:id => 2}
     assert_response :success
 
-    # membership of private project admin can see
-    assert_select 'li a', :text => "OnlineStore"
+    assert_select 'table.list.projects>tbody' do
+      assert_select 'tr:nth-of-type(1)' do
+        assert_select 'td:nth-of-type(1)>span>a', :text => 'eCookbook'
+        assert_select 'td:nth-of-type(2)', :text => 'Manager'
+      end
+      assert_select 'tr:nth-of-type(2)' do
+        assert_select 'td:nth-of-type(1)>span>a', :text => 'Private child of eCookbook'
+        assert_select 'td:nth-of-type(2)', :text => 'Manager'
+      end
+      assert_select 'tr:nth-of-type(3)' do
+        assert_select 'td:nth-of-type(1)>span>a', :text => 'OnlineStore'
+        assert_select 'td:nth-of-type(2)', :text => 'Developer'
+      end
+    end
   end
 
   def test_show_current_should_require_authentication
@@ -185,15 +275,59 @@ class UsersControllerTest < Redmine::ControllerTest
     assert_select 'h2', :text => /John Smith/
   end
 
+  def test_show_issues_counts
+    @request.session[:user_id] = 2
+    get :show, :params => {:id => 2}
+    assert_select 'table.list.issue-report>tbody' do
+      assert_select 'tr:nth-of-type(1)' do
+        assert_select 'td:nth-of-type(1)>a', :text => 'Assigned issues'
+        assert_select 'td:nth-of-type(2)>a', :text => '1'   # open
+        assert_select 'td:nth-of-type(3)>a', :text => '0'   # closed
+        assert_select 'td:nth-of-type(4)>a', :text => '1'   # total
+      end
+      assert_select 'tr:nth-of-type(2)' do
+        assert_select 'td:nth-of-type(1)>a', :text => 'Reported issues'
+        assert_select 'td:nth-of-type(2)>a', :text => '11'  # open
+        assert_select 'td:nth-of-type(3)>a', :text => '2'   # closed
+        assert_select 'td:nth-of-type(4)>a', :text => '13'  # total
+      end
+    end
+  end
+
+  def test_show_user_should_list_user_groups
+    @request.session[:user_id] = 1
+    get :show, :params => {:id => 8}
+
+    assert_select 'div#groups', 1 do
+      assert_select 'h3', :text => 'Groups'
+      assert_select 'li', 2
+      assert_select 'a[href=?]', '/groups/10/edit', :text => 'A Team'
+      assert_select 'a[href=?]', '/groups/11/edit', :text => 'B Team'
+    end
+  end
+
+  def test_show_should_list_all_emails
+    EmailAddress.create!(user_id: 3, address: 'dlopper@example.net')
+    EmailAddress.create!(user_id: 3, address: 'dlopper@example.org')
+
+    @request.session[:user_id] = 1
+    get :show, params: {id: 3}
+
+    assert_select 'li', text: /Email:/ do
+      assert_select 'a:nth-of-type(1)', text: 'dlopper@somenet.foo'
+      assert_select 'a:nth-of-type(2)', text: 'dlopper@example.net'
+      assert_select 'a:nth-of-type(3)', text: 'dlopper@example.org'
+    end
+  end
+
   def test_new
     get :new
     assert_response :success
     assert_select 'input[name=?]', 'user[login]'
+    assert_select 'label[for=?]>span.required', 'user_password', 1
   end
 
   def test_create
-    Setting.bcc_recipients = '1'
-
     assert_difference 'User.count' do
       assert_difference 'ActionMailer::Base.deliveries.size' do
         post :create, :params => {
@@ -223,7 +357,7 @@ class UsersControllerTest < Redmine::ControllerTest
 
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
-    assert_equal [user.mail], mail.bcc
+    assert_equal [user.mail], mail.to
     assert_mail_body_match 'secret', mail
   end
 
@@ -244,7 +378,8 @@ class UsersControllerTest < Redmine::ControllerTest
           'time_zone' => 'Paris',
           'comments_sorting' => 'desc',
           'warn_on_leaving_unsaved' => '0',
-          'textarea_font' => 'proportional'
+          'textarea_font' => 'proportional',
+          'history_default_tab' => 'history'
         }
       }
     end
@@ -255,6 +390,7 @@ class UsersControllerTest < Redmine::ControllerTest
     assert_equal 'desc', user.pref[:comments_sorting]
     assert_equal '0', user.pref[:warn_on_leaving_unsaved]
     assert_equal 'proportional', user.pref[:textarea_font]
+    assert_equal 'history', user.pref[:history_default_tab]
   end
 
   def test_create_with_generate_password_should_email_the_password
@@ -343,19 +479,31 @@ class UsersControllerTest < Redmine::ControllerTest
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
     assert_mail_body_match '0.0.0.0', mail
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: 'eschmoe'), mail
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: 'eschmoe'
+      ),
+      mail
+    )
     assert_select_email do
       assert_select 'a[href^=?]', 'http://localhost:3000/users', :text => 'Users'
     end
 
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
   def test_create_non_admin_should_not_send_security_notification
     ActionMailer::Base.deliveries.clear
+
     post :create, :params => {
       :user => {
         :firstname => 'Edgar',
@@ -367,14 +515,19 @@ class UsersControllerTest < Redmine::ControllerTest
         :admin => '0'
       }
     }
+
     assert_nil ActionMailer::Base.deliveries.last
   end
 
-
   def test_edit
-    get :edit, :params => {:id => 2}
+    with_settings :gravatar_enabled => '1' do
+      get :edit, :params => {:id => 2}
+    end
+
     assert_response :success
+    assert_select 'h2>a+img.gravatar'
     assert_select 'input[name=?][value=?]', 'user[login]', 'jsmith'
+    assert_select 'label[for=?]>span.required', 'user_password', 0
   end
 
   def test_edit_registered_user
@@ -387,17 +540,30 @@ class UsersControllerTest < Redmine::ControllerTest
 
   def test_edit_should_be_denied_for_anonymous
     assert User.find(6).anonymous?
+
     get :edit, :params => {:id => 6}
+
     assert_response 404
   end
 
+  def test_edit_user_with_full_text_formatting_custom_field_should_not_fail
+    field = UserCustomField.find(4)
+    field.update_attribute :text_formatting, 'full'
+
+    get :edit, :params => {:id => 2}
+
+    assert_response :success
+  end
+
   def test_update
     ActionMailer::Base.deliveries.clear
+
     put :update, :params => {
       :id => 2,
       :user => {:firstname => 'Changed', :mail_notification => 'only_assigned'},
       :pref => {:hide_mail => '1', :comments_sorting => 'desc'}
     }
+
     user = User.find(2)
     assert_equal 'Changed', user.firstname
     assert_equal 'only_assigned', user.mail_notification
@@ -422,67 +588,97 @@ class UsersControllerTest < Redmine::ControllerTest
       :id => 2,
       :user => {:group_ids => ['10']}
     }
+
     user = User.find(2)
     assert_equal [10], user.group_ids
   end
 
   def test_update_with_activation_should_send_a_notification
-    u = User.new(:firstname => 'Foo', :lastname => 'Bar', :mail => 'foo.bar@somenet.foo', :language => 'fr')
+    u = User.new(:firstname => 'Foo', :lastname => 'Bar',
+                 :mail => 'foo.bar@somenet.foo', :language => 'fr')
     u.login = 'foo'
     u.status = User::STATUS_REGISTERED
     u.save!
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
 
-    put :update, :params => {
-      :id => u.id,
-      :user => {:status => User::STATUS_ACTIVE}
-    }
+    put(
+      :update,
+      :params => {
+        :id => u.id,
+        :user => {:status => User::STATUS_ACTIVE}
+      }
+    )
+
     assert u.reload.active?
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
-    assert_equal ['foo.bar@somenet.foo'], mail.bcc
+    assert_equal ['foo.bar@somenet.foo'], mail.to
     assert_mail_body_match ll('fr', :notice_account_activated), mail
   end
 
   def test_update_with_password_change_should_send_a_notification
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
 
-    put :update, :params => {
-      :id => 2,
-      :user => {:password => 'newpass123', :password_confirmation => 'newpass123'},
-      :send_information => '1'
-    }
+    put(
+      :update,
+      :params => {
+        :id => 2,
+        :user => {
+          :password => 'newpass123',
+          :password_confirmation => 'newpass123'
+        },
+       :send_information => '1'
+      }
+    )
     u = User.find(2)
     assert u.check_password?('newpass123')
 
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
-    assert_equal [u.mail], mail.bcc
+    assert_equal [u.mail], mail.to
     assert_mail_body_match 'newpass123', mail
   end
 
+  def test_update_with_password_change_by_admin_should_send_a_security_notification
+    ActionMailer::Base.deliveries.clear
+    user = User.find_by(login: 'jsmith')
+
+    put :update, :params => {
+      :id => user.id,
+      :user => {:password => 'newpass123', :password_confirmation => 'newpass123'}
+    }
+
+    assert_equal 1, ActionMailer::Base.deliveries.size
+    mail = ActionMailer::Base.deliveries.last
+    assert_equal [user.mail], mail.to
+    assert_match 'Security notification', mail.subject
+    assert_mail_body_match 'Your password has been changed.', mail
+  end
+
   def test_update_with_generate_password_should_email_the_password
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
 
-    put :update, :params => {
-      :id => 2,
-      :user => {
-        :generate_password => '1',
-        :password => '',
-        :password_confirmation => ''
-      },
-      :send_information => '1'
-    }
+    put(
+      :update,
+      :params => {
+        :id => 2,
+        :user => {
+          :generate_password => '1',
+          :password => '',
+          :password_confirmation => ''
+        },
+        :send_information => '1'
+      }
+    )
 
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
+    u = User.find(2)
+    assert_equal [u.mail], mail.to
     m = mail_body(mail).match(/Password: ([a-zA-Z0-9]+)/)
     assert m
     password = m[1]
-    assert User.find(2).check_password?(password)
+    assert u.check_password?(password)
   end
 
   def test_update_without_generate_password_should_not_change_password
@@ -559,11 +755,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: User.find(2).login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: User.find(2).login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -579,11 +785,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -599,17 +815,27 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: User.find(2).login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: User.find(2).login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
     end
 
     # if user is already locked, destroying should not send a second mail
     # (for active admins see furtherbelow)
     ActionMailer::Base.deliveries.clear
-    delete :destroy, :params => {:id => 1}
+    delete :destroy, :params => {:id => 1, :confirm => User.find(1).login}
     assert_nil ActionMailer::Base.deliveries.last
 
   end
@@ -625,11 +851,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -648,19 +884,56 @@ class UsersControllerTest < Redmine::ControllerTest
     assert_response 404
   end
 
+  def test_update_with_blank_email_should_not_raise_exception
+    assert_no_difference 'User.count' do
+      with_settings :gravatar_enabled => '1' do
+        put :update, :params => {
+          :id => 2,
+          :user => {:mail => ''}
+        }
+      end
+    end
+    assert_response :success
+    assert_select_error /Email cannot be blank/
+  end
+
   def test_destroy
     assert_difference 'User.count', -1 do
-      delete :destroy, :params => {:id => 2}
+      delete :destroy, :params => {:id => 2, :confirm => User.find(2).login}
     end
     assert_redirected_to '/users'
     assert_nil User.find_by_id(2)
   end
 
+  def test_destroy_with_lock_param_should_lock_instead
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2, :lock => 'lock'}
+    end
+    assert_redirected_to '/users'
+    assert User.find_by_id(2).locked?
+  end
+
+  def test_destroy_should_require_confirmation
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2}
+    end
+    assert_response :success
+    assert_select '.warning', :text => /Are you sure you want to delete this user/
+  end
+
+  def test_destroy_should_require_correct_confirmation
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2, :confirm => 'wrong'}
+    end
+    assert_response :success
+    assert_select '.warning', :text => /Are you sure you want to delete this user/
+  end
+
   def test_destroy_should_be_denied_for_non_admin_users
     @request.session[:user_id] = 3
 
     assert_no_difference 'User.count' do
-      get :destroy, :params => {:id => 2}
+      delete :destroy, :params => {:id => 2, :confirm => User.find(2).login}
     end
     assert_response 403
   end
@@ -668,14 +941,16 @@ class UsersControllerTest < Redmine::ControllerTest
   def test_destroy_should_be_denied_for_anonymous
     assert User.find(6).anonymous?
     assert_no_difference 'User.count' do
-      put :destroy, :params => {:id => 6}
+      delete :destroy, :params => {:id => 6, :confirm => User.find(6).login}
     end
     assert_response 404
   end
 
   def test_destroy_should_redirect_to_back_url_param
     assert_difference 'User.count', -1 do
-      delete :destroy, :params => {:id => 2, :back_url => '/users?name=foo'}
+      delete :destroy, :params => {:id => 2,
+                                   :confirm => User.find(2).login,
+                                   :back_url => '/users?name=foo'}
     end
     assert_redirected_to '/users?name=foo'
   end
@@ -685,14 +960,47 @@ class UsersControllerTest < Redmine::ControllerTest
     user.admin = true
     user.save!
     ActionMailer::Base.deliveries.clear
-    delete :destroy, :params => {:id => user.id}
+    delete :destroy, :params => {:id => user.id, :confirm => user.login}
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.to].flatten.include?(admin.mail)
+        end
+      )
+    end
+  end
+
+  def test_destroy_without_unsubscribe_is_denied
+    user = User.find(2)
+    user.update(admin: true) # Create other admin so self can be deleted
+    @request.session[:user_id] = user.id
+    with_settings unsubscribe: 0 do
+      assert_no_difference 'User.count' do
+        delete :destroy, params: {id: user.id}
+      end
+      assert_response 422
+    end
+  end
+
+  def test_destroy_last_admin_is_denied
+    user = User.find(1)
+    @request.session[:user_id] = user.id
+    with_settings unsubscribe: 1 do
+      assert_no_difference 'User.count' do
+        delete :destroy, params: {id: user.id}
+      end
+      assert_response 422
     end
   end
 end
