@@ -1,7 +1,7 @@
-# encoding: utf-8
-#
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,20 +23,22 @@ class WikiControllerTest < Redmine::ControllerTest
   fixtures :projects, :users, :email_addresses, :roles, :members, :member_roles,
            :enabled_modules, :wikis, :wiki_pages, :wiki_contents,
            :wiki_content_versions, :attachments,
-           :issues, :issue_statuses, :trackers
+           :issues, :issue_statuses, :trackers, :watchers
 
   def setup
     User.current = nil
   end
 
   def test_show_start_page
-    get :show, :params => {:project_id => 'ecookbook'}
-    assert_response :success
+    with_settings :text_formatting => 'textile' do
+      get :show, :params => {:project_id => 'ecookbook'}
+      assert_response :success
 
-    assert_select 'h1', :text => /CookBook documentation/
-    # child_pages macro
-    assert_select 'ul.pages-hierarchy>li>a[href=?]', '/projects/ecookbook/wiki/Page_with_an_inline_image',
-      :text => 'Page with an inline image'
+      assert_select 'h1', :text => /CookBook documentation/
+      # child_pages macro
+      assert_select 'ul.pages-hierarchy>li>a[href=?]', '/projects/ecookbook/wiki/Page_with_an_inline_image',
+                    :text => 'Page with an inline image'
+    end
   end
 
   def test_export_link
@@ -46,14 +48,24 @@ class WikiControllerTest < Redmine::ControllerTest
     assert_select 'a[href=?]', '/projects/ecookbook/wiki/CookBook_documentation.txt'
   end
 
-  def test_show_page_with_name
-    get :show, :params => {:project_id => 1, :id => 'Another_page'}
+  def test_edit_sidebar_link
+    Role.anonymous.add_permission! :edit_wiki_pages
+    Role.anonymous.add_permission! :protect_wiki_pages
+    get :show, :params => {:project_id => 'ecookbook'}
     assert_response :success
+    assert_select 'a[href=?]', '/projects/ecookbook/wiki/sidebar/edit'
+  end
 
-    assert_select 'h1', :text => /Another page/
-    # Included page with an inline image
-    assert_select 'p', :text => /This is an inline image/
-    assert_select 'img[src=?][alt=?]', '/attachments/download/3/logo.gif', 'This is a logo'
+  def test_show_page_with_name
+    with_settings :text_formatting => 'textile' do
+      get :show, :params => {:project_id => 1, :id => 'Another_page'}
+      assert_response :success
+
+      assert_select 'h1', :text => /Another page/
+      # Included page with an inline image
+      assert_select 'p', :text => /This is an inline image/
+      assert_select 'img[src=?][alt=?]', '/attachments/download/3/logo.gif', 'This is a logo'
+    end
   end
 
   def test_show_old_version
@@ -113,20 +125,49 @@ class WikiControllerTest < Redmine::ControllerTest
     assert_select 'div#sidebar', :text => /Side bar content for test_show_with_sidebar/
   end
 
-  def test_show_should_display_section_edit_links
+  def test_show_should_display_watchers
     @request.session[:user_id] = 2
-    get :show, :params => {:project_id => 1, :id => 'Page with sections'}
+    page = Project.find(1).wiki.find_page('Another_page')
+    page.add_watcher User.find(2)
+    page.add_watcher Group.find(10)
+    [['1', true], ['0', false]].each do |(gravatar_enabled, is_display_gravatar)|
+      with_settings :gravatar_enabled => gravatar_enabled do
+        get :show, :params => {:project_id => 1, :id => 'Another_page'}
+      end
 
-    assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=1', 0
-    assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=2'
-    assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=3'
+      assert_select 'div#watchers ul' do
+        assert_select 'li.user-2' do
+          assert_select 'img.gravatar[title=?]', 'John Smith', is_display_gravatar
+          assert_select 'a[href="/users/2"]'
+          assert_select 'a[class*=delete]'
+        end
+        assert_select 'li.user-10' do
+          assert_select 'img.gravatar[title=?]', 'A Team', is_display_gravatar
+          assert_select 'a[href="/users/10"]', false
+          assert_select 'a[class*=delete]'
+        end
+      end
+    end
+  end
+
+  def test_show_should_display_section_edit_links
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      get :show, :params => {:project_id => 1, :id => 'Page with sections'}
+
+      assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=1', 0
+      assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=2'
+      assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=3'
+    end
   end
 
   def test_show_current_version_should_display_section_edit_links
-    @request.session[:user_id] = 2
-    get :show, :params => {:project_id => 1, :id => 'Page with sections', :version => 3}
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      get :show, :params => {:project_id => 1, :id => 'Page with sections', :version => 3}
 
-    assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=2'
+      assert_select 'a[href=?]', '/projects/ecookbook/wiki/Page_with_sections/edit?section=2'
+    end
   end
 
   def test_show_old_version_should_not_display_section_edit_links
@@ -160,6 +201,12 @@ class WikiControllerTest < Redmine::ControllerTest
     assert_select 'select[name=?] option[value="2"][selected=selected]', 'wiki_page[parent_id]'
   end
 
+  def test_show_unexistent_version_page
+    @request.session[:user_id] = 2
+    get :show, :params => {:project_id => 1, :id => 'CookBook_documentation', :version => 100}
+    assert_response 404
+  end
+
   def test_show_should_not_show_history_without_permission
     Role.anonymous.remove_permission! :view_wiki_edits
     get :show, :params => {:project_id => 1, :id => 'Page with sections', :version => 2}
@@ -176,6 +223,16 @@ class WikiControllerTest < Redmine::ControllerTest
     assert_select 'textarea[name=?]', 'content[text]'
   end
 
+  def test_show_protected_page_shoud_show_locked_badge
+    @request.session[:user_id] = 2
+
+    get :show, :params => {:project_id => 1, :id => 'CookBook_documentation'}
+
+    assert_select 'p.wiki-update-info' do
+      assert_select 'span.badge.badge-status-locked'
+    end
+  end
+
   def test_get_new
     @request.session[:user_id] = 2
 
@@ -263,6 +320,7 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_create_page_with_attachments
+    set_tmp_attachments_directory
     @request.session[:user_id] = 2
     assert_difference 'WikiPage.count' do
       assert_difference 'Attachment.count' do
@@ -303,25 +361,25 @@ class WikiControllerTest < Redmine::ControllerTest
   def test_edit_page
     @request.session[:user_id] = 2
     get :edit, :params => {:project_id => 'ecookbook', :id => 'Another_page'}
-
     assert_response :success
-
     assert_select 'textarea[name=?]', 'content[text]',
-      :text => WikiPage.find_by_title('Another_page').content.text
+                  :text => WikiPage.find_by_title('Another_page').content.text
   end
 
   def test_edit_section
-    @request.session[:user_id] = 2
-    get :edit, :params => {:project_id => 'ecookbook', :id => 'Page_with_sections', :section => 2}
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      get :edit, :params => {:project_id => 'ecookbook', :id => 'Page_with_sections', :section => 2}
 
-    assert_response :success
+      assert_response :success
 
-    page = WikiPage.find_by_title('Page_with_sections')
-    section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
+      page = WikiPage.find_by_title('Page_with_sections')
+      section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
 
-    assert_select 'textarea[name=?]', 'content[text]', :text => section
-    assert_select 'input[name=section][type=hidden][value="2"]'
-    assert_select 'input[name=section_hash][type=hidden][value=?]', hash
+      assert_select 'textarea[name=?]', 'content[text]', :text => section
+      assert_select 'input[name=section][type=hidden][value="2"]'
+      assert_select 'input[name=section_hash][type=hidden][value=?]', hash
+    end
   end
 
   def test_edit_invalid_section_should_respond_with_404
@@ -433,6 +491,7 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_update_page_with_attachments_only_should_not_create_content_version
+    set_tmp_attachments_directory
     @request.session[:user_id] = 2
     assert_no_difference 'WikiPage.count' do
       assert_no_difference 'WikiContent.count' do
@@ -457,6 +516,7 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_update_with_deleted_attachment_ids
+    set_tmp_attachments_directory
     @request.session[:user_id] = 2
     page = WikiPage.find(4)
     attachment = page.attachments.first
@@ -476,6 +536,7 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_update_with_deleted_attachment_ids_and_failure_should_preserve_selected_attachments
+    set_tmp_attachments_directory
     @request.session[:user_id] = 2
     page = WikiPage.find(4)
     attachment = page.attachments.first
@@ -544,57 +605,61 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_update_section
-    @request.session[:user_id] = 2
-    page = WikiPage.find_by_title('Page_with_sections')
-    section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
-    text = page.content.text
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      page = WikiPage.find_by(title: 'Page_with_sections')
+      section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
+      text = page.content.text
 
-    assert_no_difference 'WikiPage.count' do
-      assert_no_difference 'WikiContent.count' do
-        assert_difference 'WikiContentVersion.count' do
-          put :update, :params => {
-            :project_id => 1,
-            :id => 'Page_with_sections',
-            :content => {
-              :text => "New section content",
-              :version => 3
-            },
-            :section => 2,
-            :section_hash => hash
-          }
+      assert_no_difference 'WikiPage.count' do
+        assert_no_difference 'WikiContent.count' do
+          assert_difference 'WikiContentVersion.count' do
+            put :update, :params => {
+              :project_id => 1,
+              :id => 'Page_with_sections',
+              :content => {
+                :text => 'New section content',
+                :version => 3
+              },
+              :section => 2,
+              :section_hash => hash
+            }
+          end
         end
       end
+      assert_redirected_to '/projects/ecookbook/wiki/Page_with_sections#section-2'
+      assert_equal Redmine::WikiFormatting::Textile::Formatter.new(text).update_section(2, 'New section content'), page.reload.content.text
     end
-    assert_redirected_to '/projects/ecookbook/wiki/Page_with_sections#section-2'
-    assert_equal Redmine::WikiFormatting::Textile::Formatter.new(text).update_section(2, "New section content"), page.reload.content.text
   end
 
   def test_update_section_should_allow_stale_page_update
-    @request.session[:user_id] = 2
-    page = WikiPage.find_by_title('Page_with_sections')
-    section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
-    text = page.content.text
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      page = WikiPage.find_by(title: 'Page_with_sections')
+      section, hash = Redmine::WikiFormatting::Textile::Formatter.new(page.content.text).get_section(2)
+      text = page.content.text
 
-    assert_no_difference 'WikiPage.count' do
-      assert_no_difference 'WikiContent.count' do
-        assert_difference 'WikiContentVersion.count' do
-          put :update, :params => {
-            :project_id => 1,
-            :id => 'Page_with_sections',
-            :content => {
-              :text => "New section content",
-              :version => 2 # Current version is 3
-            },
-            :section => 2,
-            :section_hash => hash
-          }
+      assert_no_difference 'WikiPage.count' do
+        assert_no_difference 'WikiContent.count' do
+          assert_difference 'WikiContentVersion.count' do
+            put :update, :params => {
+              :project_id => 1,
+              :id => 'Page_with_sections',
+              :content => {
+                :text => 'New section content',
+                :version => 2 # Current version is 3
+              },
+              :section => 2,
+              :section_hash => hash
+            }
+          end
         end
       end
+      assert_redirected_to '/projects/ecookbook/wiki/Page_with_sections#section-2'
+      page.reload
+      assert_equal Redmine::WikiFormatting::Textile::Formatter.new(text).update_section(2, 'New section content'), page.content.text
+      assert_equal 4, page.content.version
     end
-    assert_redirected_to '/projects/ecookbook/wiki/Page_with_sections#section-2'
-    page.reload
-    assert_equal Redmine::WikiFormatting::Textile::Formatter.new(text).update_section(2, "New section content"), page.content.text
-    assert_equal 4, page.content.version
   end
 
   def test_update_section_should_not_allow_stale_section_update
@@ -624,33 +689,37 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_preview
-    @request.session[:user_id] = 2
-    post :preview, :params => {
-      :project_id => 1,
-      :id => 'CookBook_documentation',
-      :content => {
-        :comments => '',
-        :text => 'this is a *previewed text*',
-        :version => 3
-      }
-    }, :xhr => true
-    assert_response :success
-    assert_select 'strong', :text => /previewed text/
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      post :preview, :params => {
+        :project_id => 1,
+        :id => 'CookBook_documentation',
+        :content => {
+          :comments => '',
+          :text => 'this is a *previewed text*',
+          :version => 3
+        }
+      }, :xhr => true
+      assert_response :success
+      assert_select 'strong', :text => /previewed text/
+    end
   end
 
   def test_preview_new_page
-    @request.session[:user_id] = 2
-    post :preview, :params => {
-      :project_id => 1,
-      :id => 'New page',
-      :content => {
-        :text => 'h1. New page',
-        :comments => '',
-        :version => 0
-      }
-    }, :xhr => true
-    assert_response :success
-    assert_select 'h1', :text => /New page/
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      post :preview, :params => {
+        :project_id => 1,
+        :id => 'New page',
+        :content => {
+          :text => 'h1. New page',
+          :comments => '',
+          :version => 0
+        }
+      }, :xhr => true
+      assert_response :success
+      assert_select 'h1', :text => /New page/
+    end
   end
 
   def test_history
@@ -902,6 +971,7 @@ class WikiControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 2
     delete :destroy, :params => {:project_id => 1, :id => 'Child_2'}
     assert_redirected_to :action => 'index', :project_id => 'ecookbook'
+    assert_equal 'Successful deletion.', flash[:notice]
   end
 
   def test_destroy_parent_should_ask_confirmation
@@ -923,6 +993,7 @@ class WikiControllerTest < Redmine::ControllerTest
       delete :destroy, :params => {:project_id => 1, :id => 'Another_page', :todo => 'nullify'}
     end
     assert_redirected_to :action => 'index', :project_id => 'ecookbook'
+    assert_equal 'Successful deletion.', flash[:notice]
     assert_nil WikiPage.find_by_id(2)
   end
 
@@ -932,6 +1003,7 @@ class WikiControllerTest < Redmine::ControllerTest
       delete :destroy, :params => {:project_id => 1, :id => 'Another_page', :todo => 'destroy'}
     end
     assert_redirected_to :action => 'index', :project_id => 'ecookbook'
+    assert_equal 'Successful deletion.', flash[:notice]
     assert_nil WikiPage.find_by_id(2)
     assert_nil WikiPage.find_by_id(5)
   end
@@ -942,6 +1014,7 @@ class WikiControllerTest < Redmine::ControllerTest
       delete :destroy, :params => {:project_id => 1, :id => 'Another_page', :todo => 'reassign', :reassign_to_id => 1}
     end
     assert_redirected_to :action => 'index', :project_id => 'ecookbook'
+    assert_equal 'Successful deletion.', flash[:notice]
     assert_nil WikiPage.find_by_id(2)
     assert_equal WikiPage.find(1), WikiPage.find_by_id(5).parent
   end
@@ -979,11 +1052,11 @@ class WikiControllerTest < Redmine::ControllerTest
     end
 
     assert_select 'ul.pages-hierarchy' do
-      assert_select 'li' do
-        assert_select 'a[href=?]', '/projects/ecookbook/wiki/CookBook_documentation', :text => 'CookBook documentation'
+      assert_select 'li:nth-child(1) > a[href=?]', '/projects/ecookbook/wiki/Another_page', :text => 'Another page'
+      assert_select 'li:nth-child(2)' do
+        assert_select '> a[href=?]', '/projects/ecookbook/wiki/CookBook_documentation', :text => 'CookBook documentation'
         assert_select 'ul li a[href=?]', '/projects/ecookbook/wiki/Page_with_an_inline_image', :text => 'Page with an inline image'
       end
-      assert_select 'li a[href=?]', '/projects/ecookbook/wiki/Another_page', :text => 'Another page'
     end
   end
 
@@ -997,8 +1070,13 @@ class WikiControllerTest < Redmine::ControllerTest
     get :export, :params => {:project_id => 'ecookbook'}
 
     assert_response :success
-    assert_equal "text/html", @response.content_type
+    assert_equal "text/html", @response.media_type
 
+    assert_select 'ul.pages-hierarchy' do
+      assert_select 'li:nth-child(1) > a[href=?]', '#Another_page', :text => 'Another page'
+      assert_select 'li:nth-child(2) > a[href=?]', '#CookBook_documentation', :text => 'CookBook documentation'
+      assert_select 'li:nth-child(3) > a[href=?]', '#Page_with_sections', :text => 'Page with sections'
+    end
     assert_select "a[name=?]", "CookBook_documentation"
     assert_select "a[name=?]", "Another_page"
     assert_select "a[name=?]", "Page_with_an_inline_image"
@@ -1009,8 +1087,8 @@ class WikiControllerTest < Redmine::ControllerTest
     get :export, :params => {:project_id => 'ecookbook', :format => 'pdf'}
 
     assert_response :success
-    assert_equal 'application/pdf', @response.content_type
-    assert_equal 'attachment; filename="ecookbook.pdf"', @response.headers['Content-Disposition']
+    assert_equal 'application/pdf', @response.media_type
+    assert_equal "attachment; filename=\"ecookbook.pdf\"; filename*=UTF-8''ecookbook.pdf", @response.headers['Content-Disposition']
     assert @response.body.starts_with?('%PDF')
   end
 
@@ -1074,31 +1152,35 @@ class WikiControllerTest < Redmine::ControllerTest
     get :show, :params => {:project_id => 1, :format => 'pdf'}
     assert_response :success
 
-    assert_equal 'application/pdf', @response.content_type
-    assert_equal 'attachment; filename="CookBook_documentation.pdf"',
-                  @response.headers['Content-Disposition']
+    assert_equal 'application/pdf', @response.media_type
+    assert_equal "attachment; filename=\"CookBook_documentation.pdf\"; filename*=UTF-8''CookBook_documentation.pdf",
+                 @response.headers['Content-Disposition']
   end
 
   def test_show_html
-    @request.session[:user_id] = 2
-    get :show, :params => {:project_id => 1, :format => 'html'}
-    assert_response :success
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      get :show, :params => {:project_id => 1, :format => 'html'}
+      assert_response :success
 
-    assert_equal 'text/html', @response.content_type
-    assert_equal 'attachment; filename="CookBook_documentation.html"',
+      assert_equal 'text/html', @response.media_type
+      assert_equal "attachment; filename=\"CookBook_documentation.html\"; filename*=UTF-8''CookBook_documentation.html",
                   @response.headers['Content-Disposition']
-    assert_select 'h1', :text => /CookBook documentation/
+      assert_select 'h1', :text => /CookBook documentation/
+    end
   end
 
   def test_show_versioned_html
-    @request.session[:user_id] = 2
-    get :show, :params => {:project_id => 1, :format => 'html', :version => 2}
-    assert_response :success
+    with_settings :text_formatting => 'textile' do
+      @request.session[:user_id] = 2
+      get :show, :params => {:project_id => 1, :format => 'html', :version => 2}
+      assert_response :success
 
-    assert_equal 'text/html', @response.content_type
-    assert_equal 'attachment; filename="CookBook_documentation.html"',
+      assert_equal 'text/html', @response.media_type
+      assert_equal "attachment; filename=\"CookBook_documentation.html\"; filename*=UTF-8''CookBook_documentation.html",
                   @response.headers['Content-Disposition']
-    assert_select 'h1', :text => /CookBook documentation v2/
+      assert_select 'h1', :text => /CookBook documentation v2/
+    end
   end
 
   def test_show_txt
@@ -1106,9 +1188,9 @@ class WikiControllerTest < Redmine::ControllerTest
     get :show, :params => {:project_id => 1, :format => 'txt'}
     assert_response :success
 
-    assert_equal 'text/plain', @response.content_type
-    assert_equal 'attachment; filename="CookBook_documentation.txt"',
-                  @response.headers['Content-Disposition']
+    assert_equal 'text/plain', @response.media_type
+    assert_equal "attachment; filename=\"CookBook_documentation.txt\"; filename*=UTF-8''CookBook_documentation.txt",
+                 @response.headers['Content-Disposition']
     assert_include 'h1. CookBook documentation', @response.body
   end
 
@@ -1117,13 +1199,13 @@ class WikiControllerTest < Redmine::ControllerTest
     get :show, :params => {:project_id => 1, :format => 'txt', :version => 2}
     assert_response :success
 
-    assert_equal 'text/plain', @response.content_type
-    assert_equal 'attachment; filename="CookBook_documentation.txt"',
-                  @response.headers['Content-Disposition']
+    assert_equal 'text/plain', @response.media_type
+    assert_equal "attachment; filename=\"CookBook_documentation.txt\"; filename*=UTF-8''CookBook_documentation.txt",
+                 @response.headers['Content-Disposition']
     assert_include 'h1. CookBook documentation v2', @response.body
   end
 
-  def test_show_filename_should_be_uri_encoded_for_ms_browsers
+  def test_show_filename_should_be_uri_encoded
     @request.session[:user_id] = 2
     title = 'Этика_менеджмента'
     %w|pdf html txt|.each do |format|
@@ -1131,15 +1213,16 @@ class WikiControllerTest < Redmine::ControllerTest
       @request.user_agent = ""
       get :show, :params => {:project_id => 1, :id => title, :format => format}
       assert_response :success
-      assert_equal "attachment; filename=\"#{title}.#{format}\"",
-                    @response.headers['Content-Disposition']
-      # Microsoft's browsers: filename should be URI encoded
+      ascii_filename = "%3F%3F%3F%3F%3F_%3F%3F%3F%3F%3F%3F%3F%3F%3F%3F%3F.#{format}"
+      utf8_filename = "%D0%AD%D1%82%D0%B8%D0%BA%D0%B0_%D0%BC%D0%B5%D0%BD%D0%B5%D0%B4%D0%B6%D0%BC%D0%B5%D0%BD%D1%82%D0%B0.#{format}"
+      assert_equal "attachment; filename=\"#{ascii_filename}\"; filename*=UTF-8''#{utf8_filename}",
+                   @response.headers['Content-Disposition']
+      # Microsoft's browsers
       @request.user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36 Edge/15.15063'
       get :show, :params => {:project_id => 1, :id => title, :format => format}
       assert_response :success
-      filename = URI.encode("#{title}.#{format}")
-      assert_equal "attachment; filename=\"#{filename}\"",
-                    @response.headers['Content-Disposition']
+      assert_equal "attachment; filename=\"#{ascii_filename}\"; filename*=UTF-8''#{utf8_filename}",
+                   @response.headers['Content-Disposition']
     end
   end
 
@@ -1169,6 +1252,7 @@ class WikiControllerTest < Redmine::ControllerTest
   end
 
   def test_add_attachment
+    set_tmp_attachments_directory
     @request.session[:user_id] = 2
     assert_difference 'Attachment.count' do
       post :add_attachment, :params => {
