@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,7 +22,8 @@ require File.expand_path('../../../test_helper', __FILE__)
 class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   fixtures :projects, :versions, :users, :roles, :members, :member_roles, :issues, :journals, :journal_details,
            :trackers, :projects_trackers, :issue_statuses, :enabled_modules, :enumerations, :boards, :messages,
-           :attachments, :custom_fields, :custom_values, :time_entries, :issue_categories
+           :attachments, :custom_fields, :custom_values, :custom_fields_projects, :time_entries, :issue_categories,
+           :queries
 
   def setup
     super
@@ -28,31 +31,39 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   end
 
   test "GET /projects.xml should return projects" do
+    project = Project.find(1)
+    project.inherit_members = '1'
+    project.save!
+
     get '/projects.xml'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
-    assert_select 'projects>project>id', :text => '1'
-    assert_select 'projects>project>status', :text => '1'
-    assert_select 'projects>project>is_public', :text => 'true'
+    assert_select 'projects>project:first-child' do
+      assert_select '>id', :text => '1'
+      assert_select '>status', :text => '1'
+      assert_select '>is_public', :text => 'true'
+      assert_select '>inherit_members', :text => 'true'
+    end
   end
 
   test "GET /projects.json should return projects" do
     get '/projects.json'
     assert_response :success
-    assert_equal 'application/json', @response.content_type
+    assert_equal 'application/json', @response.media_type
 
     json = ActiveSupport::JSON.decode(response.body)
     assert_kind_of Hash, json
     assert_kind_of Array, json['projects']
     assert_kind_of Hash, json['projects'].first
     assert json['projects'].first.has_key?('id')
+    assert json['projects'].first.has_key?('inherit_members')
   end
 
   test "GET /projects.xml with include=issue_categories should return categories" do
     get '/projects.xml?include=issue_categories'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'issue_categories[type=array] issue_category[id="2"][name=Recipes]'
   end
@@ -60,7 +71,7 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   test "GET /projects.xml with include=trackers should return trackers" do
     get '/projects.xml?include=trackers'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'trackers[type=array] tracker[id="2"][name="Feature request"]'
   end
@@ -68,19 +79,30 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   test "GET /projects.xml with include=enabled_modules should return enabled modules" do
     get '/projects.xml?include=enabled_modules'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'enabled_modules[type=array] enabled_module[name=issue_tracking]'
   end
 
+  test "GET /projects.xml with include=issue_custom_fields should return custom fields" do
+    get '/projects.xml?include=issue_custom_fields'
+    assert_response :success
+    assert_equal 'application/xml', @response.media_type
+
+    assert_select 'issue_custom_fields[type=array] custom_field[name="Project 1 cf"]'
+  end
+
   test "GET /projects/:id.xml should return the project" do
+    Project.find(1).update!(:inherit_members => '1')
+
     get '/projects/1.xml'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'project>id', :text => '1'
     assert_select 'project>status', :text => '1'
     assert_select 'project>is_public', :text => 'true'
+    assert_select 'project>inherit_members', :text => 'true'
     assert_select 'custom_field[name="Development status"]', :text => 'Stable'
 
     assert_select 'trackers', 0
@@ -94,6 +116,7 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
     assert_kind_of Hash, json
     assert_kind_of Hash, json['project']
     assert_equal 1, json['project']['id']
+    assert_equal false, json['project']['inherit_members']
     assert_equal false, json['project'].has_key?('default_version')
     assert_equal false, json['project'].has_key?('default_assignee')
   end
@@ -103,7 +126,7 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
 
     get '/projects/1.xml'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'custom_field[name=?]', 'Development status', 0
   end
@@ -111,7 +134,7 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   test "GET /projects/:id.xml with include=issue_categories should return categories" do
     get '/projects/1.xml?include=issue_categories'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'issue_categories[type=array] issue_category[id="2"][name=Recipes]'
   end
@@ -119,7 +142,7 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   test "GET /projects/:id.xml with include=time_entry_activities should return activities" do
     get '/projects/1.xml?include=time_entry_activities'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'time_entry_activities[type=array] time_entry_activity[id="10"][name=Development]'
   end
@@ -127,15 +150,42 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
   test "GET /projects/:id.xml with include=trackers should return trackers" do
     get '/projects/1.xml?include=trackers'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'trackers[type=array] tracker[id="2"][name="Feature request"]'
   end
 
+  test "GET /projects/:id.xml with include=trackers should return trackers based on role-based permissioning" do
+    project = Project.find(1)
+    assert_equal [1, 2, 3], project.tracker_ids
+
+    role = Role.find(3) # Reporter
+    role.permissions_all_trackers = {'view_issues' => '0'}
+    role.permissions_tracker_ids = {'view_issues' => ['1']}
+    role.save!
+
+    user = User.find_by(:login => 'jsmith')
+    member = project.members.detect{|m| m.user == user}
+    member.roles.delete_all
+    member.role_ids = [role.id]
+    member.roles.reload
+    assert_equal [role.id], member.role_ids
+
+    get '/projects/1.xml?include=trackers', :headers => credentials(user.login)
+    assert_response :success
+    assert_equal 'application/xml', @response.media_type
+
+    assert_select 'trackers[type=array]' do
+      assert_select 'tracker[id="1"]', :count => 1
+      assert_select 'tracker[id="2"]', :count => 0
+      assert_select 'tracker[id="3"]', :count => 0
+    end
+  end
+
   test "GET /projects/:id.xml with include=enabled_modules should return enabled modules" do
     get '/projects/1.xml?include=enabled_modules'
     assert_response :success
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
 
     assert_select 'enabled_modules[type=array] enabled_module[name=issue_tracking]'
   end
@@ -163,12 +213,26 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
     assert_equal version.name, json['project']['default_version']['name']
   end
 
+  def test_get_project_should_not_load_default_query
+    query = ProjectQuery.find(11)
+    ProjectQuery.stubs(:default).returns query
+
+    get '/projects.json'
+
+    assert results = JSON.parse(@response.body)['projects']
+
+    assert_equal 4, results.count
+    assert results.detect{ |i| i['name'] == "eCookbook"}
+  end
+
   test "POST /projects.xml with valid parameters should create the project" do
     with_settings :default_projects_modules => ['issue_tracking', 'repository'] do
       assert_difference('Project.count') do
-        post '/projects.xml',
+        post(
+          '/projects.xml',
           :params => {:project => {:name => 'API test', :identifier => 'api-test'}},
           :headers => credentials('admin')
+        )
       end
     end
 
@@ -179,15 +243,23 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
     assert_equal Tracker.all.size, project.trackers.size
 
     assert_response :created
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'project id', :text => project.id.to_s
   end
 
   test "POST /projects.xml should accept enabled_module_names attribute" do
     assert_difference('Project.count') do
-      post '/projects.xml',
-        :params => {:project => {:name => 'API test', :identifier => 'api-test', :enabled_module_names => ['issue_tracking', 'news', 'time_tracking']}},
+      post(
+        '/projects.xml',
+        :params => {
+          :project => {
+            :name => 'API test',
+            :identifier => 'api-test',
+            :enabled_module_names => ['issue_tracking', 'news', 'time_tracking']
+          }
+        },
         :headers => credentials('admin')
+      )
     end
 
     project = Project.order('id DESC').first
@@ -196,9 +268,17 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
 
   test "POST /projects.xml should accept tracker_ids attribute" do
     assert_difference('Project.count') do
-      post '/projects.xml',
-        :params => {:project => {:name => 'API test', :identifier => 'api-test', :tracker_ids => [1, 3]}},
+      post(
+        '/projects.xml',
+        :params => {
+          :project => {
+            :name => 'API test',
+            :identifier => 'api-test',
+            :tracker_ids => [1, 3]
+          }
+        },
         :headers => credentials('admin')
+      )
     end
 
     project = Project.order('id DESC').first
@@ -207,36 +287,47 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
 
   test "POST /projects.xml with invalid parameters should return errors" do
     assert_no_difference('Project.count') do
-      post '/projects.xml',
+      post(
+        '/projects.xml',
         :params => {:project => {:name => 'API test'}},
         :headers => credentials('admin')
+      )
     end
 
     assert_response :unprocessable_entity
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'errors error', :text => "Identifier cannot be blank"
   end
 
   test "PUT /projects/:id.xml with valid parameters should update the project" do
     assert_no_difference 'Project.count' do
-      put '/projects/2.xml',
+      put(
+        '/projects/2.xml',
         :params => {:project => {:name => 'API update'}},
         :headers => credentials('jsmith')
+      )
     end
-    assert_response :ok
+    assert_response :no_content
     assert_equal '', @response.body
-    assert_equal 'application/xml', @response.content_type
+    assert_nil @response.media_type
     project = Project.find(2)
     assert_equal 'API update', project.name
   end
 
   test "PUT /projects/:id.xml should accept enabled_module_names attribute" do
     assert_no_difference 'Project.count' do
-      put '/projects/2.xml',
-        :params => {:project => {:name => 'API update', :enabled_module_names => ['issue_tracking', 'news', 'time_tracking']}},
+      put(
+        '/projects/2.xml',
+        :params => {
+          :project => {
+            :name => 'API update',
+            :enabled_module_names => ['issue_tracking', 'news', 'time_tracking']
+          }
+        },
         :headers => credentials('admin')
+      )
     end
-    assert_response :ok
+    assert_response :no_content
     assert_equal '', @response.body
     project = Project.find(2)
     assert_equal ['issue_tracking', 'news', 'time_tracking'], project.enabled_module_names.sort
@@ -244,11 +335,13 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
 
   test "PUT /projects/:id.xml should accept tracker_ids attribute" do
     assert_no_difference 'Project.count' do
-      put '/projects/2.xml',
+      put(
+        '/projects/2.xml',
         :params => {:project => {:name => 'API update', :tracker_ids => [1, 3]}},
         :headers => credentials('admin')
+      )
     end
-    assert_response :ok
+    assert_response :no_content
     assert_equal '', @response.body
     project = Project.find(2)
     assert_equal [1, 3], project.trackers.map(&:id).sort
@@ -256,22 +349,58 @@ class Redmine::ApiTest::ProjectsTest < Redmine::ApiTest::Base
 
   test "PUT /projects/:id.xml with invalid parameters should return errors" do
     assert_no_difference('Project.count') do
-      put '/projects/2.xml',
+      put(
+        '/projects/2.xml',
         :params => {:project => {:name => ''}},
         :headers => credentials('admin')
+      )
     end
 
     assert_response :unprocessable_entity
-    assert_equal 'application/xml', @response.content_type
+    assert_equal 'application/xml', @response.media_type
     assert_select 'errors error', :text => "Name cannot be blank"
   end
 
   test "DELETE /projects/:id.xml should delete the project" do
-    assert_difference('Project.count',-1) do
+    assert_difference('Project.count', -1) do
       delete '/projects/2.xml', :headers => credentials('admin')
     end
-    assert_response :ok
+    assert_response :no_content
     assert_equal '', @response.body
     assert_nil Project.find_by_id(2)
   end
+
+  test "PUT /projects/:id/archive.xml should archive project" do
+    put '/projects/1/archive.xml', :headers => credentials('admin')
+    assert_response :no_content
+    assert_equal '', @response.body
+    assert p = Project.find(1)
+    assert_not p.active?
+  end
+
+  test "PUT /projects/:id/unarchive.xml should unarchive project" do
+    Project.find(1).update_column :status, Project::STATUS_ARCHIVED
+    put '/projects/1/unarchive.xml', :headers => credentials('admin')
+    assert_response :no_content
+    assert_equal '', @response.body
+    assert p = Project.find_by_id(2)
+    assert p.active?
+  end
+
+  test "PUT /projects/:id/close.xml should close project" do
+    put '/projects/1/close.xml', :headers => credentials('admin')
+    assert_response :no_content
+    assert_equal '', @response.body
+    assert p = Project.find(1)
+    assert p.closed?
+  end
+
+  test "PUT /projects/:id/reopen.xml should reopen project" do
+    Project.find(1).update_column :status, Project::STATUS_CLOSED
+    put '/projects/1/reopen.xml', :headers => credentials('admin')
+    assert_response :no_content
+    assert_equal '', @response.body
+    assert p = Project.find(1)
+    assert p.active?
+  end
 end
