@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,8 +26,15 @@ class WelcomeTest < Redmine::IntegrationTest
   def test_robots
     get '/robots.txt'
     assert_response :success
-    assert_equal 'text/plain', @response.content_type
+    assert_equal 'text/plain', @response.media_type
     # Redmine::Utils.relative_url_root does not effect on Rails 5.1.4.
     assert @response.body.match(%r{^Disallow: /projects/ecookbook/issues\r?$})
+    assert @response.body.match(%r{^Disallow: /issues\?sort=\r?$})
+    assert @response.body.match(%r{^Disallow: /issues\?\*set_filter=\r?$})
+    assert @response.body.match(%r{^Disallow: /issues/\*\.pdf\$\r?$})
+    assert @response.body.match(%r{^Disallow: /projects/\*\.pdf\$\r?$})
+    assert @response.body.match(%r{^Disallow: /login\r?$})
+    assert @response.body.match(%r{^Disallow: /account/register\r?$})
+    assert @response.body.match(%r{^Disallow: /account/lost_password\r?$})
   end
 end
