@@ -1,7 +1,7 @@
-# encoding: utf-8
-#
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ class AttachmentTest < ActiveSupport::TestCase
            :enabled_modules, :issues, :trackers, :attachments
 
   def setup
+    User.current = nil
     set_tmp_attachments_directory
   end
 
@@ -151,6 +152,19 @@ class AttachmentTest < ActiveSupport::TestCase
     end
   end
 
+  def test_extension_update_should_be_validated_against_denied_extensions
+    with_settings :attachment_extensions_denied => "txt, png" do
+      a = Attachment.new(:container => Issue.find(1),
+                         :file => mock_file_with_options(:original_filename => "test.jpeg"),
+                         :author => User.find(1))
+      assert_save a
+
+      b = Attachment.find(a.id)
+      b.filename = "test.png"
+      assert !b.save
+    end
+  end
+
   def test_valid_extension_should_be_case_insensitive
     with_settings :attachment_extensions_allowed => "txt, Png" do
       assert Attachment.valid_extension?(".pnG")
@@ -194,11 +208,11 @@ class AttachmentTest < ActiveSupport::TestCase
     copy = a.copy
     copy.save!
 
-    assert File.exists?(diskfile)
+    assert File.exist?(diskfile)
     a.destroy
-    assert File.exists?(diskfile)
+    assert File.exist?(diskfile)
     copy.destroy
-    assert !File.exists?(diskfile)
+    assert !File.exist?(diskfile)
   end
 
   def test_create_should_auto_assign_content_type
@@ -234,6 +248,23 @@ class AttachmentTest < ActiveSupport::TestCase
     assert_not_equal a1.diskfile, a2.diskfile
   end
 
+  def test_identical_attachments_created_in_same_transaction_should_not_end_up_unreadable
+    attachments = []
+    Project.transaction do
+      3.times do
+        a = Attachment.create!(
+          :container => Issue.find(1), :author => User.find(1),
+          :file => mock_file(:filename => 'foo', :content => 'abcde')
+        )
+        attachments << a
+      end
+    end
+    attachments.each do |a|
+      assert a.readable?
+    end
+    assert_equal 1, attachments.map(&:diskfile).uniq.size
+  end
+
   def test_filename_should_be_basenamed
     a = Attachment.new(:file => mock_file(:original_filename => "path/to/the/file"))
     assert_equal 'file', a.filename
@@ -244,12 +275,32 @@ class AttachmentTest < ActiveSupport::TestCase
     assert_equal 'valid_[] invalid_chars', a.filename
   end
 
-  def test_diskfilename
-    assert Attachment.disk_filename("test_file.txt") =~ /^\d{12}_test_file.txt$/
-    assert_equal 'test_file.txt', Attachment.disk_filename("test_file.txt")[13..-1]
-    assert_equal '770c509475505f37c2b8fb6030434d6b.txt', Attachment.disk_filename("test_accentué.txt")[13..-1]
-    assert_equal 'f8139524ebb8f32e51976982cd20a85d', Attachment.disk_filename("test_accentué")[13..-1]
-    assert_equal 'cbb5b0f30978ba03731d61f9f6d10011', Attachment.disk_filename("test_accentué.ça")[13..-1]
+  def test_create_diskfile
+    path = nil
+    Attachment.create_diskfile("test_file.txt") do |f|
+      path = f.path
+      assert_match(/^\d{12}_test_file.txt$/, File.basename(path))
+      assert_equal 'test_file.txt', File.basename(path)[13..-1]
+    end
+    File.unlink path
+
+    Attachment.create_diskfile("test_accentué.txt") do |f|
+      path = f.path
+      assert_equal '770c509475505f37c2b8fb6030434d6b.txt', File.basename(f.path)[13..-1]
+    end
+    File.unlink path
+
+    Attachment.create_diskfile("test_accentué") do |f|
+      path = f.path
+      assert_equal 'f8139524ebb8f32e51976982cd20a85d', File.basename(f.path)[13..-1]
+    end
+    File.unlink path
+
+    Attachment.create_diskfile("test_accentué.ça") do |f|
+      path = f.path
+      assert_equal 'cbb5b0f30978ba03731d61f9f6d10011', File.basename(f.path)[13..-1]
+    end
+    File.unlink path
   end
 
   def test_title
@@ -277,13 +328,41 @@ class AttachmentTest < ActiveSupport::TestCase
     end
   end
 
+  def test_archive_attachments
+    attachment = Attachment.create!(:file => uploaded_test_file("testfile.txt", ""), :author_id => 1)
+    zip_data = Attachment.archive_attachments([attachment])
+    file_names = []
+    Zip::InputStream.open(StringIO.new(zip_data)) do |io|
+      while (entry = io.get_next_entry)
+        file_names << entry.name
+      end
+    end
+    assert_equal ['testfile.txt'], file_names
+  end
+
+  def test_archive_attachments_without_attachments
+    zip_data = Attachment.archive_attachments([])
+    assert_nil zip_data
+  end
+
+  def test_archive_attachments_should_rename_duplicate_file_names
+    attachment1 = Attachment.create!(:file => uploaded_test_file("testfile.txt", ""), :author_id => 1)
+    attachment2 = Attachment.create!(:file => uploaded_test_file("testfile.txt", ""), :author_id => 1)
+    zip_data = Attachment.archive_attachments([attachment1, attachment2])
+    file_names = []
+    Zip::InputStream.open(StringIO.new(zip_data)) do |io|
+      while (entry = io.get_next_entry)
+        file_names << entry.name
+      end
+    end
+    assert_equal ['testfile.txt', 'testfile(1).txt'], file_names
+  end
+
   def test_move_from_root_to_target_directory_should_move_root_files
     a = Attachment.find(20)
     assert a.disk_directory.blank?
     # Create a real file for this fixture
-    File.open(a.diskfile, "w") do |f|
-      f.write "test file at the root of files directory"
-    end
+    File.write(a.diskfile, 'test file at the root of files directory')
     assert a.readable?
     Attachment.move_from_root_to_target_directory
 
@@ -295,7 +374,8 @@ class AttachmentTest < ActiveSupport::TestCase
   test "Attachmnet.attach_files should attach the file" do
     issue = Issue.first
     assert_difference 'Attachment.count' do
-      Attachment.attach_files(issue,
+      Attachment.attach_files(
+        issue,
         '1' => {
           'file' => uploaded_test_file('testfile.txt', 'text/plain'),
           'description' => 'test'
@@ -307,7 +387,7 @@ class AttachmentTest < ActiveSupport::TestCase
     assert_equal 59, attachment.filesize
     assert_equal 'test', attachment.description
     assert_equal 'text/plain', attachment.content_type
-    assert File.exists?(attachment.diskfile)
+    assert File.exist?(attachment.diskfile)
     assert_equal 59, File.size(attachment.diskfile)
   end
 
@@ -332,7 +412,7 @@ class AttachmentTest < ActiveSupport::TestCase
     @project = Project.find(1)
     attachment = Attachment.create!(:file => uploaded_test_file("testfile.txt", ""), :author_id => 1, :created_on => 2.days.ago)
     assert_equal 'text/plain', attachment.content_type
-    Attachment.attach_files(@project, { '1' => {'token' => attachment.token } })
+    Attachment.attach_files(@project, {'1' => {'token' => attachment.token}})
     attachment.reload
     assert_equal 'text/plain', attachment.content_type
   end
@@ -343,16 +423,21 @@ class AttachmentTest < ActiveSupport::TestCase
     assert attachment.readable?
     attachment.update_digest_to_sha256!
     assert_equal 'ac5c6e99a21ae74b2e3f5b8e5b568be1b9107cd7153d139e822b9fe5caf50938', attachment.digest
+  ensure
+    set_tmp_attachments_directory
   end
 
   def test_update_attachments
     attachments = Attachment.where(:id => [2, 3]).to_a
-
-    assert Attachment.update_attachments(attachments, {
-      '2' => {:filename => 'newname.txt', :description => 'New description'},
-      3 => {:filename => 'othername.txt'}
-    })
-
+    assert(
+      Attachment.update_attachments(
+        attachments,
+        {
+          '2' => {:filename => 'newname.txt', :description => 'New description'},
+          3 => {:filename => 'othername.txt'}
+        }
+      )
+    )
     attachment = Attachment.find(2)
     assert_equal 'newname.txt', attachment.filename
     assert_equal 'New description', attachment.description
@@ -363,23 +448,29 @@ class AttachmentTest < ActiveSupport::TestCase
 
   def test_update_attachments_with_failure
     attachments = Attachment.where(:id => [2, 3]).to_a
-
-    assert !Attachment.update_attachments(attachments, {
-      '2' => {:filename => '', :description => 'New description'},
-      3 => {:filename => 'othername.txt'}
-    })
-
+    assert(
+      !Attachment.update_attachments(
+        attachments,
+        {
+          '2' => {
+            :filename => '', :description => 'New description'
+          },
+          3 => {:filename => 'othername.txt'}
+        }
+      )
+    )
     attachment = Attachment.find(3)
     assert_equal 'logo.gif', attachment.filename
   end
 
   def test_update_attachments_should_sanitize_filename
     attachments = Attachment.where(:id => 2).to_a
-
-    assert Attachment.update_attachments(attachments, {
-      2 => {:filename => 'newname?.txt'},
-    })
-
+    assert(
+      Attachment.update_attachments(
+        attachments,
+        {2 => {:filename => 'newname?.txt'},}
+      )
+    )
     attachment = Attachment.find(2)
     assert_equal 'newname_.txt', attachment.filename
   end
@@ -402,36 +493,107 @@ class AttachmentTest < ActiveSupport::TestCase
     assert_equal 17, la1.id
     la2 = Attachment.latest_attach([a1, a2], "Testfile.PNG")
     assert_equal 17, la2.id
-
+  ensure
     set_tmp_attachments_directory
   end
 
   def test_latest_attach_should_not_error_with_string_with_invalid_encoding
-    string = "width:50\xFE-Image.jpg".force_encoding('UTF-8')
+    string = "width:50\xFE-Image.jpg"
     assert_equal false, string.valid_encoding?
 
     Attachment.latest_attach(Attachment.limit(2).to_a, string)
   end
 
+  def test_latest_attach_should_support_unicode_case_folding
+    a_capital = Attachment.create!(
+      :author => User.find(1),
+      :file => mock_file(:filename => 'Ā.TXT')
+    )
+    a_small = Attachment.create!(
+      :author => User.find(1),
+      :file => mock_file(:filename => 'ā.txt')
+    )
+
+    assert_equal(a_small, Attachment.latest_attach([a_capital, a_small], 'Ā.TXT'))
+  end
+
   def test_thumbnailable_should_be_true_for_images
+    skip unless convert_installed?
     assert_equal true, Attachment.new(:filename => 'test.jpg').thumbnailable?
   end
 
-  def test_thumbnailable_should_be_true_for_non_images
+  def test_thumbnailable_should_be_false_for_images_if_convert_is_unavailable
+    Redmine::Thumbnail.stubs(:convert_available?).returns(false)
+    assert_equal false, Attachment.new(:filename => 'test.jpg').thumbnailable?
+  end
+
+  def test_thumbnailable_should_be_false_for_non_images
     assert_equal false, Attachment.new(:filename => 'test.txt').thumbnailable?
   end
 
   if convert_installed?
     def test_thumbnail_should_generate_the_thumbnail
       set_fixtures_attachments_directory
-      attachment = Attachment.find(16)
+      Attachment.clear_thumbnails
+      to_test = []
+      # image/png
+      to_test << Attachment.find(16)
+      # application/pdf
+      if Redmine::Thumbnail.gs_available?
+        to_test << Attachment.find(23)
+      else
+        puts '(Ghostscript not available)'
+      end
+
+      assert_difference "Dir.glob(File.join(Attachment.thumbnails_storage_path, '*.thumb')).size", to_test.size do
+        to_test.each do |attachment|
+          thumbnail = attachment.thumbnail
+          thumbnail_name = "#{attachment.digest}_#{attachment.filesize}_#{Setting.thumbnails_size}.thumb"
+          assert_equal thumbnail_name, File.basename(thumbnail)
+          assert File.exist?(thumbnail)
+        end
+      end
+    ensure
+      set_tmp_attachments_directory
+    end
+
+    def test_should_reuse_thumbnail
       Attachment.clear_thumbnails
 
+      a = Attachment.create!(
+        :container => Issue.find(1),
+        :file => uploaded_test_file("2010/11/101123161450_testfile_1.png", "image/png"),
+        :author => User.find(1)
+      )
+      a_thumb = b_thumb = nil
       assert_difference "Dir.glob(File.join(Attachment.thumbnails_storage_path, '*.thumb')).size" do
-        thumbnail = attachment.thumbnail
-        assert_equal "16_8e0294de2441577c529f170b6fb8f638_100.thumb", File.basename(thumbnail)
-        assert File.exists?(thumbnail)
+        a_thumb = a.thumbnail
       end
+
+      b = Attachment.create!(
+        :container => Issue.find(2),
+        :file => uploaded_test_file("2010/11/101123161450_testfile_1.png", "image/png"),
+        :author => User.find(1)
+      )
+      assert_no_difference "Dir.glob(File.join(Attachment.thumbnails_storage_path, '*.thumb')).size" do
+        b_thumb = b.thumbnail
+      end
+      assert_equal a_thumb, b_thumb
+    end
+
+    def test_destroy_should_destroy_thumbnails
+      a = Attachment.create!(
+        :container => Issue.find(1),
+        :file => uploaded_test_file("2010/11/101123161450_testfile_1.png", "image/png"),
+        :author => User.find(1)
+      )
+      diskfile  = a.diskfile
+      thumbnail = a.thumbnail
+      assert File.exist?(diskfile)
+      assert File.exist?(thumbnail)
+      assert a.destroy
+      refute File.exist?(diskfile)
+      refute File.exist?(thumbnail)
     end
 
     def test_thumbnail_should_return_nil_if_generation_fails
@@ -439,6 +601,29 @@ class AttachmentTest < ActiveSupport::TestCase
       set_fixtures_attachments_directory
       attachment = Attachment.find(16)
       assert_nil attachment.thumbnail
+    ensure
+      set_tmp_attachments_directory
+    end
+
+    def test_thumbnail_should_be_at_least_of_requested_size
+      set_fixtures_attachments_directory
+      attachment = Attachment.find(16)
+      Attachment.clear_thumbnails
+      [
+        [0, 100],
+        [49, 50],
+        [50, 50],
+        [51, 100],
+        [100, 100],
+        [101, 150],
+      ].each do |size, generated_size|
+        thumbnail = attachment.thumbnail(size: size)
+        assert_equal(
+          "8e0294de2441577c529f170b6fb8f638_2654_#{generated_size}.thumb",
+          File.basename(thumbnail))
+      end
+    ensure
+      set_tmp_attachments_directory
     end
   else
     puts '(ImageMagick convert not available)'
