@@ -1,7 +1,7 @@
-# encoding: utf-8
-#
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -75,9 +75,38 @@ class ChangesetTest < ActiveSupport::TestCase
     assert_equal [1, 2], c.issue_ids.sort
   end
 
+  def test_project_specific_activity
+    project = Project.find 1
+    activity = TimeEntryActivity.find 9
+
+    Setting.commit_ref_keywords = '*'
+    Setting.commit_logtime_enabled = '1'
+    Setting.commit_logtime_activity_id = activity.id
+
+    project_specific_activity = TimeEntryActivity.create!(
+      name: activity.name,
+      parent_id: activity.id,
+      position: activity.position,
+      project_id: project.id
+    )
+
+    c = Changeset.new(:repository   => project.repository,
+                      :committed_on => 24.hours.ago,
+                      :comments     => "Worked on this issue #1 @8h",
+                      :revision     => '520',
+                      :user         => User.find(2))
+    assert_difference 'TimeEntry.count' do
+      c.scan_comment_for_issue_ids
+    end
+
+    time = TimeEntry.order('id desc').first
+    assert_equal project_specific_activity, time.activity
+  end
+
   def test_ref_keywords_any_with_timelog
     Setting.commit_ref_keywords = '*'
     Setting.commit_logtime_enabled = '1'
+    Setting.commit_logtime_activity_id = 9
 
     {
       '2' => 2.0,
@@ -108,12 +137,16 @@ class ChangesetTest < ActiveSupport::TestCase
       assert_equal 1, time.issue_id
       assert_equal 1, time.project_id
       assert_equal 2, time.user_id
-      assert_equal expected_hours, time.hours,
-          "@#{syntax} should be logged as #{expected_hours} hours but was #{time.hours}"
+      assert_equal(
+        expected_hours, time.hours,
+        "@#{syntax} should be logged as #{expected_hours} hours but was #{time.hours}"
+      )
       assert_equal Date.yesterday, time.spent_on
-      assert time.activity.is_default?
-      assert time.comments.include?('r520'),
-            "r520 was expected in time_entry comments: #{time.comments}"
+      assert_equal 9, time.activity_id
+      assert(
+        time.comments.include?('r520'),
+        "r520 was expected in time_entry comments: #{time.comments}"
+      )
     end
   end
 
@@ -166,7 +199,7 @@ class ChangesetTest < ActiveSupport::TestCase
                       :comments     => '[#1 #2, #3] Worked on these',
                       :revision     => '12345')
     assert c.save
-    assert_equal [1,2,3], c.issue_ids.sort
+    assert_equal [1, 2, 3], c.issue_ids.sort
   end
 
   def test_ref_keywords_with_large_number_should_not_error
@@ -186,7 +219,11 @@ class ChangesetTest < ActiveSupport::TestCase
 
     with_settings :commit_update_keywords => [{'keywords' => 'fixes', 'status_id' => '3'}] do
       assert_difference 'Journal.count' do
-        c = Changeset.generate!(:repository => Project.find(1).repository,:comments => "Fixes ##{issue.id}")
+        c = Changeset.
+              generate!(
+                :repository => Project.find(1).repository,
+                :comments => "Fixes ##{issue.id}"
+              )
         assert_include c.id, issue.reload.changeset_ids
         journal = Journal.order('id DESC').first
         assert_equal 1, journal.details.count
@@ -196,10 +233,13 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_update_keywords_without_change_should_not_create_journal
     issue = Issue.generate!(:project_id => 1, :status_id => 3)
-
     with_settings :commit_update_keywords => [{'keywords' => 'fixes', 'status_id' => '3'}] do
       assert_no_difference 'Journal.count' do
-        c = Changeset.generate!(:repository => Project.find(1).repository,:comments => "Fixes ##{issue.id}")
+        c = Changeset.
+              generate!(
+                :repository => Project.find(1).repository,
+                :comments => "Fixes ##{issue.id}"
+              )
         assert_include c.id, issue.reload.changeset_ids
       end
     end
@@ -210,7 +250,6 @@ class ChangesetTest < ActiveSupport::TestCase
       {'keywords' => 'fixes, closes', 'status_id' => '5'},
       {'keywords' => 'resolves', 'status_id' => '3'}
     ] do
-
       issue1 = Issue.generate!
       issue2 = Issue.generate!
       Changeset.generate!(:comments => "Closes ##{issue1.id}\nResolves ##{issue2.id}")
@@ -224,7 +263,6 @@ class ChangesetTest < ActiveSupport::TestCase
       {'keywords' => 'fixes', 'status_id' => '5', 'if_tracker_id' => '2'},
       {'keywords' => 'fixes', 'status_id' => '3', 'if_tracker_id' => ''}
     ] do
-
       issue1 = Issue.generate!(:tracker_id => 2)
       issue2 = Issue.generate!
       Changeset.generate!(:comments => "Fixes ##{issue1.id}, ##{issue2.id}")
@@ -238,7 +276,6 @@ class ChangesetTest < ActiveSupport::TestCase
       {'keywords' => 'Fixes, Closes', 'status_id' => '5', 'done_ratio' => '100', 'if_tracker_id' => '2'},
       {'keywords' => 'Testing',       'status_id' => '3', 'done_ratio' => '90',  'if_tracker_id' => '2'}
     ] do
-
       issue1 = Issue.generate!(:tracker_id => 2)
       issue2 = Issue.generate!(:tracker_id => 2)
       Changeset.generate!(:comments => "Testing ##{issue1.id}, Fixes ##{issue2.id}")
@@ -256,7 +293,6 @@ class ChangesetTest < ActiveSupport::TestCase
       {'keywords' => 'fixes', 'status_id' => '5', 'if_tracker_id' => '2'},
       {'keywords' => 'fixes', 'status_id' => '3', 'if_tracker_id' => '3'}
     ] do
-
       issue1 = Issue.generate!(:tracker_id => 2)
       issue2 = Issue.generate!
       Changeset.generate!(:comments => "Fixes ##{issue1.id}, ##{issue2.id}")
@@ -296,9 +332,11 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_commit_referencing_a_parent_project_issue
     # repository of child project
-    r = Repository::Subversion.create!(
-          :project => Project.find(3),
-          :url     => 'svn://localhost/test')
+    r = Repository::Subversion.
+         create!(
+           :project => Project.find(3),
+           :url     => 'svn://localhost/test'
+         )
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
                       :comments     => 'refs #2, an issue of a parent project',
@@ -309,9 +347,11 @@ class ChangesetTest < ActiveSupport::TestCase
   end
 
   def test_commit_referencing_a_project_with_commit_cross_project_ref_disabled
-    r = Repository::Subversion.create!(
-          :project => Project.find(3),
-          :url     => 'svn://localhost/test')
+    r = Repository::Subversion.
+          create!(
+            :project => Project.find(3),
+            :url     => 'svn://localhost/test'
+          )
     with_settings :commit_cross_project_ref => '0' do
       c = Changeset.new(:repository   => r,
                         :committed_on => Time.now,
@@ -323,9 +363,11 @@ class ChangesetTest < ActiveSupport::TestCase
   end
 
   def test_commit_referencing_a_project_with_commit_cross_project_ref_enabled
-    r = Repository::Subversion.create!(
-          :project => Project.find(3),
-          :url     => 'svn://localhost/test')
+    r = Repository::Subversion.
+          create!(
+            :project => Project.find(3),
+            :url     => 'svn://localhost/test'
+          )
     with_settings :commit_cross_project_ref => '1' do
       c = Changeset.new(:repository   => r,
                         :committed_on => Time.now,
@@ -385,19 +427,23 @@ class ChangesetTest < ActiveSupport::TestCase
   end
 
   def test_text_tag_revision_with_repository_identifier
-    r = Repository::Subversion.create!(
-          :project_id => 1,
-          :url     => 'svn://localhost/test',
-          :identifier => 'documents')
+    r = Repository::Subversion.
+         create!(
+           :project_id => 1,
+           :url     => 'svn://localhost/test',
+           :identifier => 'documents'
+         )
     c = Changeset.new(:revision => '520', :repository => r)
     assert_equal 'documents|r520', c.text_tag
     assert_equal 'ecookbook:documents|r520', c.text_tag(Project.find(2))
   end
 
   def test_text_tag_hash
-    c = Changeset.new(
-          :scmid    => '7234cb2750b63f47bff735edc50a1c0a433c2518',
-          :revision => '7234cb2750b63f47bff735edc50a1c0a433c2518')
+    c = Changeset.
+          new(
+            :scmid    => '7234cb2750b63f47bff735edc50a1c0a433c2518',
+            :revision => '7234cb2750b63f47bff735edc50a1c0a433c2518'
+          )
     assert_equal 'commit:7234cb2750b63f47bff735edc50a1c0a433c2518', c.text_tag
   end
 
@@ -417,10 +463,13 @@ class ChangesetTest < ActiveSupport::TestCase
   end
 
   def test_text_tag_hash_with_repository_identifier
-    r = Repository::Subversion.new(
+    r =
+      Repository::Subversion.
+        new(
           :project_id => 1,
           :url     => 'svn://localhost/test',
-          :identifier => 'documents')
+          :identifier => 'documents'
+        )
     c = Changeset.new(:revision => '7234cb27', :scmid => '7234cb27', :repository => r)
     assert_equal 'commit:documents|7234cb27', c.text_tag
     assert_equal 'ecookbook:commit:documents|7234cb27', c.text_tag(Project.find(2))
@@ -448,12 +497,13 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_comments_should_be_converted_to_utf8
     proj = Project.find(3)
-    # str = File.read("#{RAILS_ROOT}/test/fixtures/encoding/iso-8859-1.txt")
-    str = "Texte encod\xe9 en ISO-8859-1.".force_encoding("ASCII-8BIT")
-    r = Repository::Bazaar.create!(
+    str = "Texte encod\xe9 en ISO-8859-1.".b
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'ISO-8859-1' )
+            :log_encoding => 'ISO-8859-1'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
@@ -461,25 +511,24 @@ class ChangesetTest < ActiveSupport::TestCase
                       :scmid        => '12345',
                       :comments     => str)
     assert( c.save )
-    str_utf8 = "Texte encod\xc3\xa9 en ISO-8859-1.".force_encoding("UTF-8")
-    assert_equal str_utf8, c.comments
+    assert_equal 'Texte encodé en ISO-8859-1.', c.comments
   end
 
   def test_invalid_utf8_sequences_in_comments_should_be_replaced_latin1
     proj = Project.find(3)
-    # str = File.read("#{RAILS_ROOT}/test/fixtures/encoding/iso-8859-1.txt")
-    str1 = "Texte encod\xe9 en ISO-8859-1.".force_encoding("UTF-8")
-    str2 = "\xe9a\xe9b\xe9c\xe9d\xe9e test".force_encoding("ASCII-8BIT")
-    r = Repository::Bazaar.create!(
+    str2 = "\xe9a\xe9b\xe9c\xe9d\xe9e test".b
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'UTF-8' )
+            :log_encoding => 'UTF-8'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
                       :revision     => '123',
                       :scmid        => '12345',
-                      :comments     => str1,
+                      :comments     => "Texte encod\xE9 en ISO-8859-1.",
                       :committer    => str2)
     assert( c.save )
     assert_equal "Texte encod? en ISO-8859-1.", c.comments
@@ -488,11 +537,13 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_invalid_utf8_sequences_in_comments_should_be_replaced_ja_jis
     proj = Project.find(3)
-    str = "test\xb5\xfetest\xb5\xfe".force_encoding('ASCII-8BIT')
-    r = Repository::Bazaar.create!(
+    str = "test\xb5\xfetest\xb5\xfe".b
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'ISO-2022-JP' )
+            :log_encoding => 'ISO-2022-JP'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
@@ -504,20 +555,22 @@ class ChangesetTest < ActiveSupport::TestCase
   end
 
   def test_comments_should_be_converted_all_latin1_to_utf8
-    s1 = "\xC2\x80"
-    s2 = "\xc3\x82\xc2\x80"
+    s1 = +"\xC2\x80"
+    s2 = +"\xc3\x82\xc2\x80"
     s4 = s2.dup
     s3 = s1.dup
-    s1.force_encoding('ASCII-8BIT')
-    s2.force_encoding('ASCII-8BIT')
+    s1 = s1.b
+    s2 = s2.b
     s3.force_encoding('ISO-8859-1')
     s4.force_encoding('UTF-8')
     assert_equal s3.encode('UTF-8'), s4
     proj = Project.find(3)
-    r = Repository::Bazaar.create!(
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'ISO-8859-1' )
+            :log_encoding => 'ISO-8859-1'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
@@ -530,26 +583,31 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_invalid_utf8_sequences_in_paths_should_be_replaced
     proj = Project.find(3)
-    str1 = "Texte encod\xe9 en ISO-8859-1".force_encoding("UTF-8")
-    str2 = "\xe9a\xe9b\xe9c\xe9d\xe9e test".force_encoding("ASCII-8BIT")
-    r = Repository::Bazaar.create!(
+    str2 = "\xe9a\xe9b\xe9c\xe9d\xe9e test".b
+    r = Repository::Bazaar.
+          create!(
             :project => proj,
             :url => '/tmp/test/bazaar',
-            :log_encoding => 'UTF-8' )
+            :log_encoding => 'UTF-8'
+          )
     assert r
-    cs = Changeset.new(
-               :repository   => r,
-               :committed_on => Time.now,
-               :revision     => '123',
-               :scmid        => '12345',
-               :comments     => "test")
+    cs = Changeset.
+           new(
+             :repository   => r,
+             :committed_on => Time.now,
+             :revision     => '123',
+             :scmid        => '12345',
+             :comments     => "test"
+           )
     assert(cs.save)
-    ch = Change.new(
-                  :changeset     => cs,
-                  :action        => "A",
-                  :path          => str1,
-                  :from_path     => str2,
-                  :from_revision => "345")
+    ch = Change.
+           new(
+             :changeset     => cs,
+             :action        => "A",
+             :path          => "Texte encod\xE9 en ISO-8859-1",
+             :from_path     => str2,
+             :from_revision => "345"
+           )
     assert(ch.save)
     assert_equal "Texte encod? en ISO-8859-1", ch.path
     assert_equal "?a?b?c?d?e test", ch.from_path
@@ -557,10 +615,12 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_comments_nil
     proj = Project.find(3)
-    r = Repository::Bazaar.create!(
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'ISO-8859-1' )
+            :log_encoding => 'ISO-8859-1'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
@@ -576,10 +636,12 @@ class ChangesetTest < ActiveSupport::TestCase
 
   def test_comments_empty
     proj = Project.find(3)
-    r = Repository::Bazaar.create!(
+    r = Repository::Bazaar.
+          create!(
             :project      => proj,
             :url          => '/tmp/test/bazaar',
-            :log_encoding => 'ISO-8859-1' )
+            :log_encoding => 'ISO-8859-1'
+          )
     assert r
     c = Changeset.new(:repository   => r,
                       :committed_on => Time.now,
