@@ -1,7 +1,7 @@
-# encoding: utf-8
-#
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,10 +24,11 @@ class MailHandlerTest < ActiveSupport::TestCase
            :members, :member_roles, :users,
            :email_addresses, :user_preferences,
            :issues, :issue_statuses,
+           :journals, :journal_details,
            :workflows, :trackers, :projects_trackers,
            :versions, :enumerations, :issue_categories,
-           :custom_fields, :custom_fields_trackers, :custom_fields_projects,
-           :boards, :messages
+           :custom_fields, :custom_fields_trackers, :custom_fields_projects, :custom_values,
+           :boards, :messages, :watchers
 
   FIXTURES_PATH = File.dirname(__FILE__) + '/../fixtures/mail_handler'
 
@@ -42,10 +43,14 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_specific_overrides
-    issue = submit_email('ticket_on_given_project.eml',
-      :allow_override => ['status', 'start_date', 'due_date', 'assigned_to',
-                          'fixed_version', 'estimated_hours', 'done_ratio', 'parent_issue']
-    )
+    issue =
+      submit_email(
+        'ticket_on_given_project.eml',
+        :allow_override =>
+          ['status', 'start_date', 'due_date', 'assigned_to',
+           'fixed_version', 'estimated_hours', 'done_ratio',
+           'parent_issue']
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -110,11 +115,12 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_to_project_specified_by_subaddress
     # This email has redmine+onlinestore@somenet.foo as 'To' header
-    issue = submit_email(
-              'ticket_on_project_given_by_to_header.eml',
-              :issue => {:tracker => 'Support request'},
-              :project_from_subaddress => 'redmine@somenet.foo'
-            )
+    issue =
+      submit_email(
+        'ticket_on_project_given_by_to_header.eml',
+        :issue => {:tracker => 'Support request'},
+        :project_from_subaddress => 'redmine@somenet.foo'
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -124,10 +130,11 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_with_default_tracker
     # This email contains: 'Project: onlinestore'
-    issue = submit_email(
-              'ticket_on_given_project.eml',
-              :issue => {:tracker => 'Support request'}
-            )
+    issue =
+      submit_email(
+        'ticket_on_given_project.eml',
+        :issue => {:tracker => 'Support request'}
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -136,10 +143,11 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_with_default_version
     # This email contains: 'Project: onlinestore'
-    issue = submit_email(
-              'ticket_on_given_project.eml',
-              :issue => {:fixed_version => 'Alpha'}
-            )
+    issue =
+      submit_email(
+        'ticket_on_given_project.eml',
+        :issue => {:fixed_version => 'Alpha'}
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     assert_equal 'Alpha', issue.reload.fixed_version.name
@@ -147,10 +155,11 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_with_default_assigned_to
     # This email contains: 'Project: onlinestore'
-    issue = submit_email(
-              'ticket_on_given_project.eml',
-              :issue => {:assigned_to => 'jsmith'}
-            )
+    issue =
+      submit_email(
+        'ticket_on_given_project.eml',
+        :issue => {:assigned_to => 'jsmith'}
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     assert_equal 'jsmith', issue.reload.assigned_to.login
@@ -186,11 +195,12 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_partial_attributes_override
-    issue = submit_email(
-              'ticket_with_attributes.eml',
-              :issue => {:priority => 'High'},
-              :allow_override => ['tracker']
-            )
+    issue =
+      submit_email(
+        'ticket_with_attributes.eml',
+        :issue => {:priority => 'High'},
+        :allow_override => ['tracker']
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -204,10 +214,11 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_spaces_between_attribute_and_separator
-    issue = submit_email(
-              'ticket_with_spaces_between_attribute_and_separator.eml',
-              :allow_override => 'tracker,category,priority'
-            )
+    issue =
+      submit_email(
+        'ticket_with_spaces_between_attribute_and_separator.eml',
+        :allow_override => 'tracker,category,priority'
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -241,9 +252,12 @@ class MailHandlerTest < ActiveSupport::TestCase
                                          :name => 'OS', :multiple => true,
                                          :possible_values => ['Linux', 'Windows', 'Mac OS X'])
 
-    issue = submit_email('ticket_with_custom_fields.eml',
-      :issue => {:project => 'onlinestore'}, :allow_override => ['database', 'Searchable_field', 'OS']
-    )
+    issue =
+      submit_email(
+        'ticket_with_custom_fields.eml',
+        :issue => {:project => 'onlinestore'},
+        :allow_override => ['database', 'Searchable_field', 'OS']
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -258,13 +272,15 @@ class MailHandlerTest < ActiveSupport::TestCase
     field = IssueCustomField.create!(:name => 'Affected version',
                                      :field_format => 'version',
                                      :is_for_all => true,
-                                     :tracker_ids => [1,2,3])
-
-    issue = submit_email('ticket_with_custom_fields.eml',
-      :issue => {:project => 'ecookbook'}, :allow_override => ['affected version']
-    ) do |email|
-      email << "Affected version: 1.0\n"
-    end
+                                     :tracker_ids => [1, 2, 3])
+    issue =
+      submit_email(
+        'ticket_with_custom_fields.eml',
+        :issue => {:project => 'ecookbook'},
+        :allow_override => ['affected version']
+      ) do |email|
+        email << "Affected version: 1.0\n"
+      end
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -312,11 +328,13 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_by_unknown_user
     assert_no_difference 'User.count' do
-      assert_equal false,
-                   submit_email(
-                     'ticket_by_unknown_user.eml',
-                     :issue => {:project => 'ecookbook'}
-                   )
+      assert_equal(
+        false,
+        submit_email(
+          'ticket_by_unknown_user.eml',
+          :issue => {:project => 'ecookbook'}
+        )
+      )
     end
   end
 
@@ -324,11 +342,12 @@ class MailHandlerTest < ActiveSupport::TestCase
     Role.anonymous.add_permission!(:add_issues)
     Role.anonymous.add_permission!(:add_issue_watchers)
     assert_no_difference 'User.count' do
-      issue = submit_email(
-                'ticket_by_unknown_user.eml',
-                :issue => {:project => 'ecookbook'},
-                :unknown_user => 'accept'
-              )
+      issue =
+        submit_email(
+          'ticket_by_unknown_user.eml',
+          :issue => {:project => 'ecookbook'},
+          :unknown_user => 'accept'
+        )
       assert issue.is_a?(Issue)
       assert issue.author.anonymous?
       issue.reload
@@ -340,11 +359,12 @@ class MailHandlerTest < ActiveSupport::TestCase
   def test_add_issue_by_anonymous_user_with_no_from_address
     Role.anonymous.add_permission!(:add_issues)
     assert_no_difference 'User.count' do
-      issue = submit_email(
-                'ticket_by_empty_user.eml',
-                :issue => {:project => 'ecookbook'},
-                :unknown_user => 'accept'
-              )
+      issue =
+        submit_email(
+          'ticket_by_empty_user.eml',
+          :issue => {:project => 'ecookbook'},
+          :unknown_user => 'accept'
+        )
       assert issue.is_a?(Issue)
       assert issue.author.anonymous?
     end
@@ -354,12 +374,14 @@ class MailHandlerTest < ActiveSupport::TestCase
     Role.anonymous.add_permission!(:add_issues)
     assert_no_difference 'User.count' do
       assert_no_difference 'Issue.count' do
-        assert_equal false,
-                     submit_email(
-                       'ticket_by_unknown_user.eml',
-                       :issue => {:project => 'onlinestore'},
-                       :unknown_user => 'accept'
-                     )
+        assert_equal(
+          false,
+          submit_email(
+            'ticket_by_unknown_user.eml',
+            :issue => {:project => 'onlinestore'},
+            :unknown_user => 'accept'
+          )
+        )
       end
     end
   end
@@ -367,12 +389,13 @@ class MailHandlerTest < ActiveSupport::TestCase
   def test_add_issue_by_anonymous_user_on_private_project_without_permission_check
     assert_no_difference 'User.count' do
       assert_difference 'Issue.count' do
-        issue = submit_email(
-                  'ticket_by_unknown_user.eml',
-                  :issue => {:project => 'onlinestore'},
-                  :no_permission_check => '1',
-                  :unknown_user => 'accept'
-                )
+        issue =
+          submit_email(
+            'ticket_by_unknown_user.eml',
+            :issue => {:project => 'onlinestore'},
+            :no_permission_check => '1',
+            :unknown_user => 'accept'
+          )
         assert issue.is_a?(Issue)
         assert issue.author.anonymous?
         assert !issue.project.is_public?
@@ -380,14 +403,44 @@ class MailHandlerTest < ActiveSupport::TestCase
     end
   end
 
+  def test_no_issue_on_closed_project_without_permission_check
+    Project.find(2).close
+    assert_no_difference 'User.count' do
+      assert_no_difference 'Issue.count' do
+        submit_email(
+          'ticket_by_unknown_user.eml',
+          :issue => {:project => 'onlinestore'},
+          :no_permission_check => '1',
+          :unknown_user => 'accept'
+        )
+      end
+    end
+  ensure
+    Project.find(2).reopen
+  end
+
+  def test_no_issue_on_closed_project_without_issue_tracking_module
+    assert_no_difference 'User.count' do
+      assert_no_difference 'Issue.count' do
+        submit_email(
+          'ticket_by_unknown_user.eml',
+          :issue => {:project => 'subproject2'},
+          :no_permission_check => '1',
+          :unknown_user => 'accept'
+        )
+      end
+    end
+  end
+
   def test_add_issue_by_created_user
     Setting.default_language = 'en'
     assert_difference 'User.count' do
-      issue = submit_email(
-                'ticket_by_unknown_user.eml',
-                :issue => {:project => 'ecookbook'},
-                :unknown_user => 'create'
-              )
+      issue =
+        submit_email(
+          'ticket_by_unknown_user.eml',
+          :issue => {:project => 'ecookbook'},
+          :unknown_user => 'create'
+        )
       assert issue.is_a?(Issue)
       assert issue.author.active?
       assert_equal 'john.doe@somenet.foo', issue.author.mail
@@ -409,10 +462,19 @@ class MailHandlerTest < ActiveSupport::TestCase
     assert issue.is_a?(Issue)
     assert !issue.new_record?
 
-    mail = ActionMailer::Base.deliveries.last
-    assert_not_nil mail
-    assert mail.subject.include?("##{issue.id}")
-    assert mail.subject.include?('New ticket on a given project')
+    assert_equal 4, issue.parent_issue_id
+    assert_equal 2, ActionMailer::Base.deliveries.size
+
+    [
+      [issue.id, 'New ticket on a given project'],
+      [4, 'Issue on project 2'],
+    ].each do |issue_id, issue_subject|
+      mail =
+        ActionMailer::Base.deliveries.detect do |m|
+          /##{issue_id}/.match?(m.subject) && /#{issue_subject}/.match?(m.subject)
+        end
+      assert_not_nil mail
+    end
   end
 
   def test_created_user_should_be_added_to_groups
@@ -468,10 +530,11 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_add_issue_with_invalid_attributes
     with_settings :default_issue_start_date_to_creation_date => '0' do
-      issue = submit_email(
-                'ticket_with_invalid_attributes.eml',
-                :allow_override => 'tracker,category,priority'
-              )
+      issue =
+        submit_email(
+          'ticket_with_invalid_attributes.eml',
+          :allow_override => 'tracker,category,priority'
+        )
       assert issue.is_a?(Issue)
       assert !issue.new_record?
       issue.reload
@@ -486,22 +549,42 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_invalid_project_should_be_assigned_to_default_project
-    issue = submit_email('ticket_on_given_project.eml',
-                         :issue => {:project => 'ecookbook'},
-                         :allow_override => 'project') do |email|
-      email.gsub!(/^Project:.+$/, 'Project: invalid')
-    end
+    issue =
+      submit_email(
+        'ticket_on_given_project.eml',
+        :issue => {:project => 'ecookbook'},
+        :allow_override => 'project'
+      ) do |email|
+        email.gsub!(/^Project:.+$/, 'Project: invalid')
+      end
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     assert_equal 'ecookbook', issue.project.identifier
   end
 
+  def test_add_issue_with_private_keyword
+    User.find_by_mail('jsmith@somenet.foo').update_attribute 'language', 'fr'
+    # give the user permission to set issues private:
+    MemberRole.create! member_id: 3, role_id: 1
+    issue =
+      submit_email(
+        'ticket_with_localized_private_flag.eml',
+        :allow_override => 'is_private,tracker,category,priority'
+      )
+    assert issue.is_a?(Issue)
+    assert_not issue.new_record?
+    issue.reload
+    assert_equal 'New ticket on a given project', issue.subject
+    assert issue.is_private
+  end
+
   def test_add_issue_with_localized_attributes
     User.find_by_mail('jsmith@somenet.foo').update_attribute 'language', 'fr'
-    issue = submit_email(
-              'ticket_with_localized_attributes.eml',
-              :allow_override => 'tracker,category,priority'
-            )
+    issue =
+      submit_email(
+        'ticket_with_localized_attributes.eml',
+        :allow_override => 'tracker,category,priority'
+      )
     assert issue.is_a?(Issue)
     assert !issue.new_record?
     issue.reload
@@ -515,23 +598,25 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_japanese_keywords
-    ja_dev = "\xe9\x96\x8b\xe7\x99\xba".force_encoding('UTF-8')
-    tracker = Tracker.generate!(:name => ja_dev)
+    tracker = Tracker.generate!(:name => '開発')
     Project.find(1).trackers << tracker
-    issue = submit_email(
-              'japanese_keywords_iso_2022_jp.eml',
-              :issue => {:project => 'ecookbook'},
-              :allow_override => 'tracker'
-            )
+    issue =
+      submit_email(
+        'japanese_keywords_iso_2022_jp.eml',
+        :issue => {:project => 'ecookbook'},
+        :allow_override => 'tracker'
+      )
     assert_kind_of Issue, issue
     assert_equal tracker, issue.tracker
   end
 
   def test_add_issue_from_apple_mail
-    issue = submit_email(
-              'apple_mail_with_attachment.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'apple_mail_with_attachment.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
 
@@ -544,15 +629,16 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_thunderbird_with_attachment_ja
-    issue = submit_email(
-              'thunderbird_with_attachment_ja.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'thunderbird_with_attachment_ja.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
-    ja = "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88.txt".force_encoding('UTF-8')
     attachment = issue.attachments.first
-    assert_equal ja, attachment.filename
+    assert_equal 'テスト.txt', attachment.filename
     assert_equal 5, attachment.filesize
     assert File.exist?(attachment.diskfile)
     assert_equal 5, File.size(attachment.diskfile)
@@ -560,25 +646,26 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_invalid_utf8
-    issue = submit_email(
-              'invalid_utf8.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'invalid_utf8.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    description = "\xD0\x97\xD0\xB4\xD1\x80\xD0\xB0\xD0\xB2\xD1\x81\xD1\x82\xD0\xB2\xD1\x83\xD0\xB9\xD1\x82\xD0\xB5?".force_encoding('UTF-8')
-    assert_equal description, issue.description
+    assert_equal 'Здравствуйте?', issue.description
   end
 
   def test_gmail_with_attachment_ja
-    issue = submit_email(
-              'gmail_with_attachment_ja.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'gmail_with_attachment_ja.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
-    ja = "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88.txt".force_encoding('UTF-8')
     attachment = issue.attachments.first
-    assert_equal ja, attachment.filename
+    assert_equal 'テスト.txt', attachment.filename
     assert_equal 5, attachment.filesize
     assert File.exist?(attachment.diskfile)
     assert_equal 5, File.size(attachment.diskfile)
@@ -586,15 +673,17 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_thunderbird_with_attachment_latin1
-    issue = submit_email(
-              'thunderbird_with_attachment_iso-8859-1.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'thunderbird_with_attachment_iso-8859-1.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
-    u = "".force_encoding('UTF-8')
-    u1 = "\xc3\x84\xc3\xa4\xc3\x96\xc3\xb6\xc3\x9c\xc3\xbc".force_encoding('UTF-8')
-    11.times { u << u1 }
+    u = +''
+    u1 = 'ÄäÖöÜü'
+    11.times {u << u1}
     attachment = issue.attachments.first
     assert_equal "#{u}.png", attachment.filename
     assert_equal 130, attachment.filesize
@@ -604,15 +693,17 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_gmail_with_attachment_latin1
-    issue = submit_email(
-              'gmail_with_attachment_iso-8859-1.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'gmail_with_attachment_iso-8859-1.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
-    u = "".force_encoding('UTF-8')
-    u1 = "\xc3\x84\xc3\xa4\xc3\x96\xc3\xb6\xc3\x9c\xc3\xbc".force_encoding('UTF-8')
-    11.times { u << u1 }
+    u = +''
+    u1 = 'ÄäÖöÜü'
+    11.times {u << u1}
     attachment = issue.attachments.first
     assert_equal "#{u}.txt", attachment.filename
     assert_equal 5, attachment.filesize
@@ -622,10 +713,12 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_mail_with_attachment_latin2
-    issue = submit_email(
-              'ticket_with_text_attachment_iso-8859-2.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    set_tmp_attachments_directory
+    issue =
+      submit_email(
+        'ticket_with_text_attachment_iso-8859-2.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_equal 1, issue.attachments.size
     attachment = issue.attachments.first
@@ -633,15 +726,16 @@ class MailHandlerTest < ActiveSupport::TestCase
     assert_equal 19, attachment.filesize
     assert File.exist?(attachment.diskfile)
     assert_equal 19, File.size(attachment.diskfile)
-    content = "p\xF8\xEDli\xB9 \xBEluou\xE8k\xFD k\xF9n".force_encoding('CP852')
+    content = (+"p\xF8\xEDli\xB9 \xBEluou\xE8k\xFD k\xF9n").force_encoding('CP852')
     assert_equal content, File.read(attachment.diskfile).force_encoding('CP852')
   end
 
   def test_empty_attachment_should_not_be_imported
-    issue = submit_email(
-              'ticket_with_empty_attachment.eml',
-              issue: { project: 'ecookbook' }
-            )
+    issue =
+      submit_email(
+        'ticket_with_empty_attachment.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_equal 0, issue.attachments.size
   end
 
@@ -662,6 +756,17 @@ class MailHandlerTest < ActiveSupport::TestCase
     assert_equal '', issue.description
   end
 
+  def test_preferred_body_part_setting
+    with_settings :mail_handler_preferred_body_part => 'plain' do
+      issue = submit_email('different_contents_in_text_and_html.eml', :issue => {:project => 'ecookbook'})
+      assert_equal 'The text part.', issue.description
+    end
+    with_settings :mail_handler_preferred_body_part => 'html' do
+      issue = submit_email('different_contents_in_text_and_html.eml', :issue => {:project => 'ecookbook'})
+      assert_equal 'The html part.', issue.description
+    end
+  end
+
   def test_attachment_text_part_should_be_added_as_issue_attachment
     issue = submit_email('multiple_text_parts.eml', :issue => {:project => 'ecookbook'})
     assert_not_include 'Plain text attachment', issue.description
@@ -671,86 +776,87 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_add_issue_with_iso_8859_1_subject
-    issue = submit_email(
-              'subject_as_iso-8859-1.eml',
-              :issue => {:project => 'ecookbook'}
-            )
-    str = "Testmail from Webmail: \xc3\xa4 \xc3\xb6 \xc3\xbc...".force_encoding('UTF-8')
+    issue =
+      submit_email(
+        'subject_as_iso-8859-1.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    assert_equal str, issue.subject
+    assert_equal 'Testmail from Webmail: ä ö ü...', issue.subject
   end
 
   def test_quoted_printable_utf8
-    issue = submit_email(
-              'quoted_printable_utf8.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'quoted_printable_utf8.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    str = "Freundliche Gr\xc3\xbcsse".force_encoding('UTF-8')
-    assert_equal str, issue.description
+    assert_equal 'Freundliche Grüsse', issue.description
   end
 
   def test_gmail_iso8859_2
-    issue = submit_email(
-              'gmail-iso8859-2.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'gmail-iso8859-2.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    str = "Na \xc5\xa1triku se su\xc5\xa1i \xc5\xa1osi\xc4\x87.".force_encoding('UTF-8')
-    assert issue.description.include?(str)
+    assert issue.description.include?('Na štriku se suši šosić.')
   end
 
   def test_add_issue_with_japanese_subject
-    issue = submit_email(
-              'subject_japanese_1.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'subject_japanese_1.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    ja = "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88".force_encoding('UTF-8')
-    assert_equal ja, issue.subject
+    assert_equal 'テスト', issue.subject
   end
 
   def test_add_issue_with_korean_body
-    # Make sure mail bodies with a charset unknown to Ruby
-    # but known to the Mail gem 2.5.4 are handled correctly
-    kr = "\xEA\xB3\xA0\xEB\xA7\x99\xEC\x8A\xB5\xEB\x8B\x88\xEB\x8B\xA4.".force_encoding('UTF-8')
-    issue = submit_email(
-            'body_ks_c_5601-1987.eml',
-            :issue => {:project => 'ecookbook'}
-          )
+    issue =
+      submit_email(
+        'body_ks_c_5601-1987.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    assert_equal kr, issue.description
+    assert_equal '고맙습니다.', issue.description
   end
 
   def test_add_issue_with_no_subject_header
-    issue = submit_email(
-              'no_subject_header.eml',
-              :issue => {:project => 'ecookbook'}
-            )
-    assert_kind_of Issue, issue
-    assert_equal '(no subject)', issue.subject
+    with_settings :default_language => 'en' do
+      issue =
+        submit_email(
+          'no_subject_header.eml',
+          :issue => {:project => 'ecookbook'}
+        )
+      assert_kind_of Issue, issue
+      assert_equal "(no subject)", issue.subject
+    end
   end
 
   def test_add_issue_with_mixed_japanese_subject
-    issue = submit_email(
-              'subject_japanese_2.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'subject_japanese_2.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
-    ja = "Re: \xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88".force_encoding('UTF-8')
-    assert_equal ja, issue.subject
+    assert_equal 'Re: テスト', issue.subject
   end
 
   def test_add_issue_with_iso_2022_jp_ms_subject
-    # The original subject is "① 丸数字テスト".
     # CIRCLED DIGIT ONE character is undefined in ISO-2022-JP but
     # defined in some vendor-extended variants such as ISO-2022-JP-MS.
     # This test makes sure that mail gem replaces an undefined characters
     # with a replacement character instead of breaking the whole subject.
-    issue = submit_email(
-              'subject_japanese_3.eml',
-              :issue => {:project => 'ecookbook'}
-            )
+    issue =
+      submit_email(
+        'subject_japanese_3.eml',
+        :issue => {:project => 'ecookbook'}
+      )
     assert_kind_of Issue, issue
     assert_match /丸数字テスト/, issue.subject
   end
@@ -774,12 +880,14 @@ class MailHandlerTest < ActiveSupport::TestCase
     emission_addresses.each do |addr|
       with_settings :mail_from => addr do
         assert_no_difference 'User.count' do
-          assert_equal false,
-                      submit_email(
-                        'ticket_from_emission_address.eml',
-                        :issue => {:project => 'ecookbook'},
-                        :unknown_user => 'create'
-                      )
+          assert_equal(
+            false,
+            submit_email(
+              'ticket_from_emission_address.eml',
+              :issue => {:project => 'ecookbook'},
+              :unknown_user => 'create'
+            )
+          )
         end
       end
     end
@@ -921,10 +1029,11 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_update_issue_should_not_set_defaults
-    journal = submit_email(
-                'ticket_reply.eml',
-                :issue => {:tracker => 'Support request', :priority => 'High'}
-              )
+    journal =
+      submit_email(
+        'ticket_reply.eml',
+        :issue => {:tracker => 'Support request', :priority => 'High'}
+      )
     assert journal.is_a?(Journal)
     assert_match /This is reply/, journal.notes
     assert_equal 'Feature request', journal.issue.tracker.name
@@ -969,6 +1078,42 @@ class MailHandlerTest < ActiveSupport::TestCase
     end
   end
 
+  def test_reply_to_a_nonexistent_issue
+    set_tmp_attachments_directory
+    Issue.find(2).destroy
+    assert_no_difference 'Issue.count' do
+      assert_no_difference 'Journal.count' do
+        journal = submit_email('ticket_reply_with_status.eml')
+        assert_nil journal
+      end
+    end
+  end
+
+  def test_reply_to_an_issue_without_permission
+    set_tmp_attachments_directory
+    # "add_issue_notes" permission is explicit required to allow users to add notes
+    # "edit_issue" permission no longer includes the "add_issue_notes" permission
+    Role.all.each {|r| r.remove_permission! :add_issue_notes}
+    assert_no_difference 'Issue.count' do
+      assert_no_difference 'Journal.count' do
+        assert_not submit_email('ticket_reply_with_status.eml')
+      end
+    end
+  end
+
+  def test_reply_to_a_nonexitent_journal
+    journal_id = Issue.find(2).journals.last.id
+    Journal.destroy(journal_id)
+    assert_no_difference 'Issue.count' do
+      assert_no_difference 'Journal.count' do
+        journal = submit_email('ticket_reply.eml') do |email|
+          email.sub! %r{^In-Reply-To:.*$}, "In-Reply-To: <redmine.journal-#{journal_id}.20060719210421@osiris>"
+        end
+        assert_nil journal
+      end
+    end
+  end
+
   def test_reply_to_a_message
     m = submit_email('message_reply.eml')
     assert m.is_a?(Message)
@@ -999,6 +1144,21 @@ class MailHandlerTest < ActiveSupport::TestCase
     end
   end
 
+  def test_reply_to_a_nonexistent_topic
+    Message.find(2).destroy
+    assert_no_difference('Message.count') do
+      m = submit_email('message_reply_by_subject.eml')
+      assert_nil m
+    end
+  end
+
+  def test_reply_to_a_topic_without_permission
+    Role.all.each {|r| r.remove_permission! :add_messages}
+    assert_no_difference('Message.count') do
+      assert_not submit_email('message_reply_by_subject.eml')
+    end
+  end
+
   def test_should_convert_tags_of_html_only_emails
     with_settings :text_formatting => 'textile' do
       issue = submit_email('ticket_html_only.eml', :issue => {:project => 'ecookbook'})
@@ -1023,9 +1183,12 @@ class MailHandlerTest < ActiveSupport::TestCase
     assert issue.is_a?(Issue)
     issue.reload
     assert_equal 'Test email', issue.subject
-    assert_equal "Simple, unadorned test email generated by Outlook 2010. It is in HTML format, but" +
-      " no special formatting has been chosen. I’m going to save this as a draft and then manually" +
-      " drop it into the Inbox for scraping by Redmine 3.0.2.", issue.description
+    assert_equal(
+      "Simple, unadorned test email generated by Outlook 2010. It is in HTML format, but" \
+        " no special formatting has been chosen. I’m going to save this as a draft and then manually" \
+        " drop it into the Inbox for scraping by Redmine 3.0.2.",
+      issue.description
+    )
   end
 
   test "truncate emails with no setting should add the entire email into the issue" do
@@ -1043,7 +1206,7 @@ class MailHandlerTest < ActiveSupport::TestCase
       assert_issue_created(issue)
       assert issue.description.include?('This paragraph is before delimiters')
       assert issue.description.include?('--- This line starts with a delimiter')
-      assert !issue.description.match(/^---#{"\u00A0"}$/)
+      assert !issue.description.match(/^---\u00A0$/)
       assert !issue.description.include?('This paragraph is after the delimiter')
     end
   end
@@ -1100,7 +1263,7 @@ class MailHandlerTest < ActiveSupport::TestCase
   end
 
   def test_attachments_that_match_mail_handler_excluded_filenames_should_be_ignored
-    with_settings :mail_handler_excluded_filenames => '*.vcf, *.jpg' do
+    with_settings :mail_handler_excluded_filenames => "*.vcf,\n *.jpg" do
       issue = submit_email('ticket_with_attachment.eml', :issue => {:project => 'onlinestore'})
       assert issue.is_a?(Issue)
       assert !issue.new_record?
@@ -1130,7 +1293,13 @@ class MailHandlerTest < ActiveSupport::TestCase
   def test_email_with_long_subject_line
     issue = submit_email('ticket_with_long_subject.eml')
     assert issue.is_a?(Issue)
-    assert_equal issue.subject, 'New ticket on a given project with a very long subject line which exceeds 255 chars and should not be ignored but chopped off. And if the subject line is still not long enough, we just add more text. And more text. Wow, this is really annoying. Especially, if you have nothing to say...'[0,255]
+    str =
+      'New ticket on a given project with a very long subject line' \
+        ' which exceeds 255 chars and should not be ignored but chopped off.' \
+        ' And if the subject line is still not long enough, we just add more text.' \
+        ' And more text. Wow, this is really annoying.' \
+        ' Especially, if you have nothing to say...'
+    assert_equal issue.subject, str[0, 255]
   end
 
   def test_first_keyword_should_be_matched
@@ -1179,40 +1348,73 @@ class MailHandlerTest < ActiveSupport::TestCase
 
   def test_new_user_with_utf8_encoded_fullname_should_be_decoded
     assert_difference 'User.count' do
-      issue = submit_email(
-                'fullname_of_sender_as_utf8_encoded.eml',
-                :issue => {:project => 'ecookbook'},
-                :unknown_user => 'create'
-              )
+      issue =
+        submit_email(
+          'fullname_of_sender_as_utf8_encoded.eml',
+          :issue => {:project => 'ecookbook'},
+          :unknown_user => 'create'
+        )
     end
     user = User.order('id DESC').first
     assert_equal "foo@example.org", user.mail
-    str1 = "\xc3\x84\xc3\xa4".force_encoding('UTF-8')
-    str2 = "\xc3\x96\xc3\xb6".force_encoding('UTF-8')
-    assert_equal str1, user.firstname
-    assert_equal str2, user.lastname
+    assert_equal 'Ää', user.firstname
+    assert_equal 'Öö', user.lastname
+  end
+
+  def test_new_user_with_fullname_in_parentheses
+    assert_difference 'User.count' do
+      issue =
+        submit_email(
+          'fullname_of_sender_in_parentheses.eml',
+          :issue => {:project => 'ecookbook'},
+          :unknown_user => 'create'
+        )
+    end
+    user = User.order('id DESC').first
+    assert_equal "jdoe@example.net", user.mail
+    assert_equal 'John', user.firstname
+    assert_equal 'Doe', user.lastname
   end
 
   def test_extract_options_from_env_should_return_options
-    options = MailHandler.extract_options_from_env({
-      'tracker' => 'defect',
-      'project' => 'foo',
-      'unknown_user' => 'create',
-      'no_notification' => '1'
-    })
-
-    assert_equal({
-      :issue => {:tracker => 'defect', :project => 'foo'},
-      :unknown_user => 'create', :no_notification => '1'
-    }, options)
+    options =
+      MailHandler.extract_options_from_env(
+        {
+          'tracker' => 'defect',
+          'project' => 'foo',
+          'unknown_user' => 'create',
+          'no_notification' => '1'
+        }
+      )
+    assert_equal(
+      {:issue => {:tracker => 'defect', :project => 'foo'},
+       :unknown_user => 'create', :no_notification => '1'},
+      options
+    )
   end
 
   def test_safe_receive_should_rescue_exceptions_and_return_false
-    MailHandler.stubs(:receive).raises(Exception.new "Something went wrong")
+    MailHandler.stubs(:receive).raises(StandardError.new("Something went wrong"))
 
     assert_equal false, MailHandler.safe_receive
   end
 
+  def test_smine_signature
+    issue = submit_email('smime_signature.eml', :issue => {:project => 'onlinestore'})
+    assert issue.is_a?(Issue)
+    assert !issue.new_record?
+    issue.reload
+    assert_equal 'Self-Signed S/MIME signature', issue.subject
+    assert_equal User.find_by_login('jsmith'), issue.author
+    assert_equal Project.find(2), issue.project
+    assert_equal 'smime.sh.txt describes how to create Self-Signed S/MIME Certs.', issue.description
+    assert_equal 2, issue.attachments.size
+    assert_equal 'smime.sh.txt', issue.attachments[0].filename
+    assert_equal 'text/plain', issue.attachments[0].content_type
+    assert_equal 'smime.p7s', issue.attachments[1].filename
+    assert_equal 'application/x-pkcs7-signature', issue.attachments[1].content_type
+  end
+
   private
 
   def submit_email(filename, options={})
