@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,28 +25,27 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
   include Redmine::I18n
 
   REPOSITORY_PATH = Rails.root.join('tmp/test/mercurial_repository').to_s
-  NUM_REV = 34
-
-  CHAR_1_HEX = "\xc3\x9c".force_encoding('UTF-8')
-  BRANCH_CHAR_1 = "branch-#{CHAR_1_HEX}-01".force_encoding('UTF-8')
+  NUM_REV = 43
 
   def setup
     User.current = nil
     @project    = Project.find(3)
-    @repository = Repository::Mercurial.create(
-                      :project => @project,
-                      :url     => REPOSITORY_PATH,
-                      :path_encoding => 'ISO-8859-1'
-                      )
+    @repository =
+      Repository::Mercurial.create(
+        :project => @project,
+        :url     => REPOSITORY_PATH,
+        :path_encoding => 'ISO-8859-1'
+      )
     assert @repository
   end
 
   def test_blank_path_to_repository_error_message
     set_language_if_valid 'en'
-    repo = Repository::Mercurial.new(
-                          :project      => @project,
-                          :identifier   => 'test'
-                        )
+    repo =
+      Repository::Mercurial.new(
+        :project      => @project,
+        :identifier   => 'test'
+      )
     assert !repo.save
     assert_include "Path to repository cannot be blank",
                    repo.errors.full_messages
@@ -52,15 +53,15 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
 
   def test_blank_path_to_repository_error_message_fr
     set_language_if_valid 'fr'
-    str = "Chemin du d\xc3\xa9p\xc3\xb4t doit \xc3\xaatre renseign\xc3\xa9(e)".force_encoding('UTF-8')
-    repo = Repository::Mercurial.new(
-                          :project      => @project,
-                          :url          => "",
-                          :identifier   => 'test',
-                          :path_encoding => ''
-                        )
+    repo =
+      Repository::Mercurial.new(
+        :project      => @project,
+        :url          => "",
+        :identifier   => 'test',
+        :path_encoding => ''
+      )
     assert !repo.save
-    assert_include str, repo.errors.full_messages
+    assert_include 'Chemin du dépôt doit être renseigné(e)', repo.errors.full_messages
   end
 
   if File.directory?(REPOSITORY_PATH)
@@ -169,14 +170,14 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      assert_equal 46, @repository.filechanges.count
+      assert_equal 53, @repository.filechanges.count
       rev0 = @repository.changesets.find_by_revision('0')
       assert_equal "Initial import.\nThe repository contains 3 files.",
                    rev0.comments
       assert_equal "0885933ad4f68d77c2649cd11f8311276e7ef7ce", rev0.scmid
       first_rev = @repository.changesets.first
       last_rev  = @repository.changesets.last
-      assert_equal "#{NUM_REV - 1}", first_rev.revision
+      assert_equal (NUM_REV - 1).to_s, first_rev.revision
       assert_equal "0", last_rev.revision
     end
 
@@ -255,24 +256,28 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
 
       # with_limit
       changesets = @repository.latest_changesets('', nil, 2)
-      assert_equal ["#{NUM_REV - 1}", "#{NUM_REV - 2}"], changesets.collect(&:revision)
+      assert_equal [(NUM_REV - 1).to_s, (NUM_REV - 2).to_s], changesets.collect(&:revision)
 
       # with_filepath
-      changesets = @repository.latest_changesets(
-                      '/sql_escape/percent%dir/percent%file1.txt', nil)
+      changesets =
+        @repository.latest_changesets(
+          '/sql_escape/percent%dir/percent%file1.txt', nil
+        )
       assert_equal %w|30 11 10 9|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets(
-                      '/sql_escape/underscore_dir/understrike_file.txt', nil)
+      changesets =
+        @repository.latest_changesets(
+          '/sql_escape/underscore_dir/understrike_file.txt', nil
+        )
       assert_equal %w|30 12 9|, changesets.collect(&:revision)
 
       changesets = @repository.latest_changesets('README', nil)
       assert_equal %w|31 30 28 17 8 6 1 0|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets('README','8')
+      changesets = @repository.latest_changesets('README', '8')
       assert_equal %w|8 6 1 0|, changesets.collect(&:revision)
 
-      changesets = @repository.latest_changesets('README','8', 2)
+      changesets = @repository.latest_changesets('README', '8', 2)
       assert_equal %w|8 6|, changesets.collect(&:revision)
 
       # with_dirpath
@@ -354,11 +359,11 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       assert_equal NUM_REV, @repository.changesets.count
 
       if @repository.scm.class.client_version_above?([1, 6])
-        changesets = @repository.latest_changesets('', BRANCH_CHAR_1)
+        changesets = @repository.latest_changesets('', 'branch-Ü-01')
         assert_equal %w|27 26|, changesets.collect(&:revision)
       end
 
-      changesets = @repository.latest_changesets("latin-1-dir/test-#{CHAR_1_HEX}-subdir", BRANCH_CHAR_1)
+      changesets = @repository.latest_changesets('latin-1-dir/test-Ü-subdir', 'branch-Ü-01')
       assert_equal %w|27|, changesets.collect(&:revision)
     end
 
@@ -421,8 +426,8 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       scmid3 = scmid_for_assert(hex3, is_short_scmid)
       assert_equal 1, c3.size
       assert_equal 'A', c3[0].action
-      assert_equal "/latin-1-dir/test-#{CHAR_1_HEX}-1.txt",  c3[0].path
-      assert_equal "/latin-1-dir/test-#{CHAR_1_HEX}.txt",    c3[0].from_path
+      assert_equal '/latin-1-dir/test-Ü-1.txt',  c3[0].path
+      assert_equal '/latin-1-dir/test-Ü.txt',    c3[0].from_path
       assert_equal scmid3, c3[0].from_revision
     end
     private :assert_copied_files
@@ -582,7 +587,7 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       %w|27 7bbf4c738e71 7bbf|.each do |r2|
         changeset = @repository.find_changeset_by_name(r2)
         %w|28 3ae45e2d177d 3ae45|.each do |r1|
-        assert_equal @repository.find_changeset_by_name(r1), changeset.next
+          assert_equal @repository.find_changeset_by_name(r1), changeset.next
         end
       end
     end
@@ -592,7 +597,7 @@ class RepositoryMercurialTest < ActiveSupport::TestCase
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      ["#{NUM_REV - 1}", "2e6d54642923", "2e6d5"].each do |r1|
+      [(NUM_REV - 1).to_s, "ba20ebce08db", "ba20e"].each do |r1|
         changeset = @repository.find_changeset_by_name(r1)
         assert_nil changeset.next
       end
